#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains custom typing aliases."""
from pathlib import Path
from typing import IO, TYPE_CHECKING, Any, Dict, List, Optional, Tuple, TypeVar, Union

if TYPE_CHECKING:
    from telegram import InputFile
    from telegram.utils.helpers import DefaultValue

FileLike = Union[IO, 'InputFile']
"""Either an open file handler or a :class:`telegram.InputFile`."""

FileInput = Union[str, bytes, FileLike, Path]
"""Valid input for passing files to Telegram. Either a file id as string, a file like object,
a local file path as string, :class:`pathlib.Path` or the file contents as :obj:`bytes`."""

JSONDict = Dict[str, Any]
"""Dictionary containing response from Telegram or data to send to the API."""

ConversationDict = Dict[Tuple[int, ...], Optional[object]]
"""Dicts as maintained by the :class:`telegram.ext.ConversationHandler`."""

DVType = TypeVar('DVType')
ODVInput = Optional[Union['DefaultValue[DVType]', DVType]]
"""Generic type for bot method parameters which can have defaults. ``ODVInput[type]`` is the same
as ``Optional[Union[DefaultValue, type]]``."""
DVInput = Union['DefaultValue[DVType]', DVType]
"""Generic type for bot method parameters which can have defaults. ``DVInput[type]`` is the same
as ``Union[DefaultValue, type]``."""

RT = TypeVar("RT")
SLT = Union[RT, List[RT], Tuple[RT, ...]]
"""Single instance or list/tuple of instances."""



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest
from flaky import flaky

from telegram import ForceReply, ReplyKeyboardRemove


@pytest.fixture(scope='class')
def force_reply():
    return ForceReply(TestForceReply.force_reply, TestForceReply.selective)


class TestForceReply:
    force_reply = True
    selective = True

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_send_message_with_force_reply(self, bot, chat_id, force_reply):
        message = bot.send_message(chat_id, 'text', reply_markup=force_reply)

        assert message.text == 'text'

    def test_expected(self, force_reply):
        assert force_reply.force_reply == self.force_reply
        assert force_reply.selective == self.selective

    def test_to_dict(self, force_reply):
        force_reply_dict = force_reply.to_dict()

        assert isinstance(force_reply_dict, dict)
        assert force_reply_dict['force_reply'] == force_reply.force_reply
        assert force_reply_dict['selective'] == force_reply.selective

    def test_equality(self):
        a = ForceReply(True, False)
        b = ForceReply(False, False)
        c = ForceReply(True, True)
        d = ReplyKeyboardRemove()

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import os
from pathlib import Path

import pytest
from flaky import flaky

from telegram import Video, TelegramError, Voice, PhotoSize, MessageEntity, Bot
from telegram.error import BadRequest
from telegram.utils.helpers import escape_markdown
from tests.conftest import check_shortcut_call, check_shortcut_signature, check_defaults_handling


@pytest.fixture(scope='function')
def video_file():
    f = open('tests/data/telegram.mp4', 'rb')
    yield f
    f.close()


@pytest.fixture(scope='class')
def video(bot, chat_id):
    with open('tests/data/telegram.mp4', 'rb') as f:
        return bot.send_video(chat_id, video=f, timeout=50).video


class TestVideo:
    width = 360
    height = 640
    duration = 5
    file_size = 326534
    mime_type = 'video/mp4'
    supports_streaming = True
    file_name = 'telegram.mp4'

    thumb_width = 180
    thumb_height = 320
    thumb_file_size = 1767

    caption = '<b>VideoTest</b> - *Caption*'
    video_file_url = 'https://python-telegram-bot.org/static/testfiles/telegram.mp4'

    video_file_id = '5a3128a4d2a04750b5b58397f3b5e812'
    video_file_unique_id = 'adc3145fd2e84d95b64d68eaa22aa33e'

    def test_creation(self, video):
        # Make sure file has been uploaded.
        assert isinstance(video, Video)
        assert isinstance(video.file_id, str)
        assert isinstance(video.file_unique_id, str)
        assert video.file_id != ''
        assert video.file_unique_id != ''

        assert isinstance(video.thumb, PhotoSize)
        assert isinstance(video.thumb.file_id, str)
        assert isinstance(video.thumb.file_unique_id, str)
        assert video.thumb.file_id != ''
        assert video.thumb.file_unique_id != ''

    def test_expected_values(self, video):
        assert video.width == self.width
        assert video.height == self.height
        assert video.duration == self.duration
        assert video.file_size == self.file_size
        assert video.mime_type == self.mime_type

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_send_all_args(self, bot, chat_id, video_file, video, thumb_file):
        message = bot.send_video(
            chat_id,
            video_file,
            duration=self.duration,
            caption=self.caption,
            supports_streaming=self.supports_streaming,
            disable_notification=False,
            width=video.width,
            height=video.height,
            parse_mode='Markdown',
            thumb=thumb_file,
        )

        assert isinstance(message.video, Video)
        assert isinstance(message.video.file_id, str)
        assert isinstance(message.video.file_unique_id, str)
        assert message.video.file_id != ''
        assert message.video.file_unique_id != ''
        assert message.video.width == video.width
        assert message.video.height == video.height
        assert message.video.duration == video.duration
        assert message.video.file_size == video.file_size

        assert message.caption == self.caption.replace('*', '')

        assert message.video.thumb.file_size == self.thumb_file_size
        assert message.video.thumb.width == self.thumb_width
        assert message.video.thumb.height == self.thumb_height

        assert message.video.file_name == self.file_name

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_send_video_custom_filename(self, bot, chat_id, video_file, monkeypatch):
        def make_assertion(url, data, **kwargs):
            return data['video'].filename == 'custom_filename'

        monkeypatch.setattr(bot.request, 'post', make_assertion)

        assert bot.send_video(chat_id, video_file, filename='custom_filename')

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_get_and_download(self, bot, video):
        new_file = bot.get_file(video.file_id)

        assert new_file.file_size == self.file_size
        assert new_file.file_id == video.file_id
        assert new_file.file_unique_id == video.file_unique_id
        assert new_file.file_path.startswith('https://')

        new_file.download('telegram.mp4')

        assert os.path.isfile('telegram.mp4')

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_send_mp4_file_url(self, bot, chat_id, video):
        message = bot.send_video(chat_id, self.video_file_url, caption=self.caption)

        assert isinstance(message.video, Video)
        assert isinstance(message.video.file_id, str)
        assert isinstance(message.video.file_unique_id, str)
        assert message.video.file_id != ''
        assert message.video.file_unique_id != ''
        assert message.video.width == video.width
        assert message.video.height == video.height
        assert message.video.duration == video.duration
        assert message.video.file_size == video.file_size

        assert isinstance(message.video.thumb, PhotoSize)
        assert isinstance(message.video.thumb.file_id, str)
        assert isinstance(message.video.thumb.file_unique_id, str)
        assert message.video.thumb.file_id != ''
        assert message.video.thumb.file_unique_id != ''
        assert message.video.thumb.width == 51  # This seems odd that it's not self.thumb_width
        assert message.video.thumb.height == 90  # Ditto
        assert message.video.thumb.file_size == 645  # same

        assert message.caption == self.caption

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_send_video_caption_entities(self, bot, chat_id, video):
        test_string = 'Italic Bold Code'
        entities = [
            MessageEntity(MessageEntity.ITALIC, 0, 6),
            MessageEntity(MessageEntity.ITALIC, 7, 4),
            MessageEntity(MessageEntity.ITALIC, 12, 4),
        ]
        message = bot.send_video(chat_id, video, caption=test_string, caption_entities=entities)

        assert message.caption == test_string
        assert message.caption_entities == entities

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_resend(self, bot, chat_id, video):
        message = bot.send_video(chat_id, video.file_id)

        assert message.video == video

    def test_send_with_video(self, monkeypatch, bot, chat_id, video):
        def test(url, data, **kwargs):
            return data['video'] == video.file_id

        monkeypatch.setattr(bot.request, 'post', test)
        message = bot.send_video(chat_id, video=video)
        assert message

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    @pytest.mark.parametrize('default_bot', [{'parse_mode': 'Markdown'}], indirect=True)
    def test_send_video_default_parse_mode_1(self, default_bot, chat_id, video):
        test_string = 'Italic Bold Code'
        test_markdown_string = '_Italic_ *Bold* `Code`'

        message = default_bot.send_video(chat_id, video, caption=test_markdown_string)
        assert message.caption_markdown == test_markdown_string
        assert message.caption == test_string

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    @pytest.mark.parametrize('default_bot', [{'parse_mode': 'Markdown'}], indirect=True)
    def test_send_video_default_parse_mode_2(self, default_bot, chat_id, video):
        test_markdown_string = '_Italic_ *Bold* `Code`'

        message = default_bot.send_video(
            chat_id, video, caption=test_markdown_string, parse_mode=None
        )
        assert message.caption == test_markdown_string
        assert message.caption_markdown == escape_markdown(test_markdown_string)

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    @pytest.mark.parametrize('default_bot', [{'parse_mode': 'Markdown'}], indirect=True)
    def test_send_video_default_parse_mode_3(self, default_bot, chat_id, video):
        test_markdown_string = '_Italic_ *Bold* `Code`'

        message = default_bot.send_video(
            chat_id, video, caption=test_markdown_string, parse_mode='HTML'
        )
        assert message.caption == test_markdown_string
        assert message.caption_markdown == escape_markdown(test_markdown_string)

    def test_send_video_local_files(self, monkeypatch, bot, chat_id):
        # For just test that the correct paths are passed as we have no local bot API set up
        test_flag = False
        expected = (Path.cwd() / 'tests/data/telegram.jpg/').as_uri()
        file = 'tests/data/telegram.jpg'

        def make_assertion(_, data, *args, **kwargs):
            nonlocal test_flag
            test_flag = data.get('video') == expected and data.get('thumb') == expected

        monkeypatch.setattr(bot, '_post', make_assertion)
        bot.send_video(chat_id, file, thumb=file)
        assert test_flag

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    @pytest.mark.parametrize(
        'default_bot,custom',
        [
            ({'allow_sending_without_reply': True}, None),
            ({'allow_sending_without_reply': False}, None),
            ({'allow_sending_without_reply': False}, True),
        ],
        indirect=['default_bot'],
    )
    def test_send_video_default_allow_sending_without_reply(
        self, default_bot, chat_id, video, custom
    ):
        reply_to_message = default_bot.send_message(chat_id, 'test')
        reply_to_message.delete()
        if custom is not None:
            message = default_bot.send_video(
                chat_id,
                video,
                allow_sending_without_reply=custom,
                reply_to_message_id=reply_to_message.message_id,
            )
            assert message.reply_to_message is None
        elif default_bot.defaults.allow_sending_without_reply:
            message = default_bot.send_video(
                chat_id, video, reply_to_message_id=reply_to_message.message_id
            )
            assert message.reply_to_message is None
        else:
            with pytest.raises(BadRequest, match='message not found'):
                default_bot.send_video(
                    chat_id, video, reply_to_message_id=reply_to_message.message_id
                )

    def test_de_json(self, bot):
        json_dict = {
            'file_id': self.video_file_id,
            'file_unique_id': self.video_file_unique_id,
            'width': self.width,
            'height': self.height,
            'duration': self.duration,
            'mime_type': self.mime_type,
            'file_size': self.file_size,
            'file_name': self.file_name,
        }
        json_video = Video.de_json(json_dict, bot)

        assert json_video.file_id == self.video_file_id
        assert json_video.file_unique_id == self.video_file_unique_id
        assert json_video.width == self.width
        assert json_video.height == self.height
        assert json_video.duration == self.duration
        assert json_video.mime_type == self.mime_type
        assert json_video.file_size == self.file_size
        assert json_video.file_name == self.file_name

    def test_to_dict(self, video):
        video_dict = video.to_dict()

        assert isinstance(video_dict, dict)
        assert video_dict['file_id'] == video.file_id
        assert video_dict['file_unique_id'] == video.file_unique_id
        assert video_dict['width'] == video.width
        assert video_dict['height'] == video.height
        assert video_dict['duration'] == video.duration
        assert video_dict['mime_type'] == video.mime_type
        assert video_dict['file_size'] == video.file_size
        assert video_dict['file_name'] == video.file_name

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_error_send_empty_file(self, bot, chat_id):
        with pytest.raises(TelegramError):
            bot.send_video(chat_id, open(os.devnull, 'rb'))

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_error_send_empty_file_id(self, bot, chat_id):
        with pytest.raises(TelegramError):
            bot.send_video(chat_id, '')

    def test_error_without_required_args(self, bot, chat_id):
        with pytest.raises(TypeError):
            bot.send_video(chat_id=chat_id)

    def test_get_file_instance_method(self, monkeypatch, video):
        def make_assertion(*_, **kwargs):
            return kwargs['file_id'] == video.file_id

        assert check_shortcut_signature(Video.get_file, Bot.get_file, ['file_id'], [])
        assert check_shortcut_call(video.get_file, video.bot, 'get_file')
        assert check_defaults_handling(video.get_file, video.bot)

        monkeypatch.setattr(video.bot, 'get_file', make_assertion)
        assert video.get_file()

    def test_equality(self, video):
        a = Video(video.file_id, video.file_unique_id, self.width, self.height, self.duration)
        b = Video('', video.file_unique_id, self.width, self.height, self.duration)
        c = Video(video.file_id, video.file_unique_id, 0, 0, 0)
        d = Video('', '', self.width, self.height, self.duration)
        e = Voice(video.file_id, video.file_unique_id, self.duration)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import ShippingAddress


@pytest.fixture(scope='class')
def shipping_address():
    return ShippingAddress(
        TestShippingAddress.country_code,
        TestShippingAddress.state,
        TestShippingAddress.city,
        TestShippingAddress.street_line1,
        TestShippingAddress.street_line2,
        TestShippingAddress.post_code,
    )


class TestShippingAddress:
    country_code = 'GB'
    state = 'state'
    city = 'London'
    street_line1 = '12 Grimmauld Place'
    street_line2 = 'street_line2'
    post_code = 'WC1'

    def test_de_json(self, bot):
        json_dict = {
            'country_code': self.country_code,
            'state': self.state,
            'city': self.city,
            'street_line1': self.street_line1,
            'street_line2': self.street_line2,
            'post_code': self.post_code,
        }
        shipping_address = ShippingAddress.de_json(json_dict, bot)

        assert shipping_address.country_code == self.country_code
        assert shipping_address.state == self.state
        assert shipping_address.city == self.city
        assert shipping_address.street_line1 == self.street_line1
        assert shipping_address.street_line2 == self.street_line2
        assert shipping_address.post_code == self.post_code

    def test_to_dict(self, shipping_address):
        shipping_address_dict = shipping_address.to_dict()

        assert isinstance(shipping_address_dict, dict)
        assert shipping_address_dict['country_code'] == shipping_address.country_code
        assert shipping_address_dict['state'] == shipping_address.state
        assert shipping_address_dict['city'] == shipping_address.city
        assert shipping_address_dict['street_line1'] == shipping_address.street_line1
        assert shipping_address_dict['street_line2'] == shipping_address.street_line2
        assert shipping_address_dict['post_code'] == shipping_address.post_code

    def test_equality(self):
        a = ShippingAddress(
            self.country_code,
            self.state,
            self.city,
            self.street_line1,
            self.street_line2,
            self.post_code,
        )
        b = ShippingAddress(
            self.country_code,
            self.state,
            self.city,
            self.street_line1,
            self.street_line2,
            self.post_code,
        )
        d = ShippingAddress(
            '', self.state, self.city, self.street_line1, self.street_line2, self.post_code
        )
        d2 = ShippingAddress(
            self.country_code, '', self.city, self.street_line1, self.street_line2, self.post_code
        )
        d3 = ShippingAddress(
            self.country_code, self.state, '', self.street_line1, self.street_line2, self.post_code
        )
        d4 = ShippingAddress(
            self.country_code, self.state, self.city, '', self.street_line2, self.post_code
        )
        d5 = ShippingAddress(
            self.country_code, self.state, self.city, self.street_line1, '', self.post_code
        )
        d6 = ShippingAddress(
            self.country_code, self.state, self.city, self.street_line1, self.street_line2, ''
        )

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != d
        assert hash(a) != hash(d)

        assert a != d2
        assert hash(a) != hash(d2)

        assert a != d3
        assert hash(a) != hash(d3)

        assert a != d4
        assert hash(a) != hash(d4)

        assert a != d5
        assert hash(a) != hash(d5)

        assert a != d6
        assert hash(6) != hash(d6)



#!/usr/bin/env python
#
# Module author:
# Tymofii A. Khodniev (thodnev) <thodnev@mail.ru>
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/]
"""A throughput-limiting message processor for Telegram bots."""
import functools
import queue as q
import threading
import time
import warnings
from typing import TYPE_CHECKING, Callable, List, NoReturn

from telegram.ext.utils.promise import Promise
from telegram.utils.deprecate import TelegramDeprecationWarning

if TYPE_CHECKING:
    from telegram import Bot

# We need to count < 1s intervals, so the most accurate timer is needed
curtime = time.perf_counter


class DelayQueueError(RuntimeError):
    """Indicates processing errors."""


class DelayQueue(threading.Thread):
    """
    Processes callbacks from queue with specified throughput limits. Creates a separate thread to
    process callbacks with delays.

    .. deprecated:: 13.3
       :class:`telegram.ext.DelayQueue` in its current form is deprecated and will be reinvented
       in a future release. See `this thread <https://git.io/JtDbF>`_ for a list of known bugs.

    Args:
        queue (:obj:`Queue`, optional): Used to pass callbacks to thread. Creates ``Queue``
            implicitly if not provided.
        burst_limit (:obj:`int`, optional): Number of maximum callbacks to process per time-window
            defined by :attr:`time_limit_ms`. Defaults to 30.
        time_limit_ms (:obj:`int`, optional): Defines width of time-window used when each
            processing limit is calculated. Defaults to 1000.
        exc_route (:obj:`callable`, optional): A callable, accepting 1 positional argument; used to
            route exceptions from processor thread to main thread; is called on `Exception`
            subclass exceptions. If not provided, exceptions are routed through dummy handler,
            which re-raises them.
        autostart (:obj:`bool`, optional): If :obj:`True`, processor is started immediately after
            object's creation; if :obj:`False`, should be started manually by `start` method.
            Defaults to :obj:`True`.
        name (:obj:`str`, optional): Thread's name. Defaults to ``'DelayQueue-N'``, where N is
            sequential number of object created.

    Attributes:
        burst_limit (:obj:`int`): Number of maximum callbacks to process per time-window.
        time_limit (:obj:`int`): Defines width of time-window used when each processing limit is
            calculated.
        exc_route (:obj:`callable`): A callable, accepting 1 positional argument; used to route
            exceptions from processor thread to main thread;
        name (:obj:`str`): Thread's name.

    """

    _instcnt = 0  # instance counter

    def __init__(
        self,
        queue: q.Queue = None,
        burst_limit: int = 30,
        time_limit_ms: int = 1000,
        exc_route: Callable[[Exception], None] = None,
        autostart: bool = True,
        name: str = None,
    ):
        warnings.warn(
            'DelayQueue in its current form is deprecated and will be reinvented in a future '
            'release. See https://git.io/JtDbF for a list of known bugs.',
            category=TelegramDeprecationWarning,
        )

        self._queue = queue if queue is not None else q.Queue()
        self.burst_limit = burst_limit
        self.time_limit = time_limit_ms / 1000
        self.exc_route = exc_route if exc_route is not None else self._default_exception_handler
        self.__exit_req = False  # flag to gently exit thread
        self.__class__._instcnt += 1
        if name is None:
            name = f'{self.__class__.__name__}-{self.__class__._instcnt}'
        super().__init__(name=name)
        self.daemon = False
        if autostart:  # immediately start processing
            super().start()

    def run(self) -> None:
        """
        Do not use the method except for unthreaded testing purposes, the method normally is
        automatically called by autostart argument.

        """

        times: List[float] = []  # used to store each callable processing time
        while True:
            item = self._queue.get()
            if self.__exit_req:
                return  # shutdown thread
            # delay routine
            now = time.perf_counter()
            t_delta = now - self.time_limit  # calculate early to improve perf.
            if times and t_delta > times[-1]:
                # if last call was before the limit time-window
                # used to impr. perf. in long-interval calls case
                times = [now]
            else:
                # collect last in current limit time-window
                times = [t for t in times if t >= t_delta]
                times.append(now)
            if len(times) >= self.burst_limit:  # if throughput limit was hit
                time.sleep(times[1] - t_delta)
            # finally process one
            try:
                func, args, kwargs = item
                func(*args, **kwargs)
            except Exception as exc:  # re-route any exceptions
                self.exc_route(exc)  # to prevent thread exit

    def stop(self, timeout: float = None) -> None:
        """Used to gently stop processor and shutdown its thread.

        Args:
            timeout (:obj:`float`): Indicates maximum time to wait for processor to stop and its
                thread to exit. If timeout exceeds and processor has not stopped, method silently
                returns. :attr:`is_alive` could be used afterwards to check the actual status.
                ``timeout`` set to :obj:`None`, blocks until processor is shut down.
                Defaults to :obj:`None`.

        """

        self.__exit_req = True  # gently request
        self._queue.put(None)  # put something to unfreeze if frozen
        super().join(timeout=timeout)

    @staticmethod
    def _default_exception_handler(exc: Exception) -> NoReturn:
        """
        Dummy exception handler which re-raises exception in thread. Could be possibly overwritten
        by subclasses.

        """

        raise exc

    def __call__(self, func: Callable, *args: object, **kwargs: object) -> None:
        """Used to process callbacks in throughput-limiting thread through queue.

        Args:
            func (:obj:`callable`): The actual function (or any callable) that is processed through
                queue.
            *args (:obj:`list`): Variable-length `func` arguments.
            **kwargs (:obj:`dict`): Arbitrary keyword-arguments to `func`.

        """

        if not self.is_alive() or self.__exit_req:
            raise DelayQueueError('Could not process callback in stopped thread')
        self._queue.put((func, args, kwargs))


# The most straightforward way to implement this is to use 2 sequential delay
# queues, like on classic delay chain schematics in electronics.
# So, message path is:
# msg --> group delay if group msg, else no delay --> normal msg delay --> out
# This way OS threading scheduler cares of timings accuracy.
# (see time.time, time.clock, time.perf_counter, time.sleep @ docs.python.org)
class MessageQueue:
    """
    Implements callback processing with proper delays to avoid hitting Telegram's message limits.
    Contains two ``DelayQueue``, for group and for all messages, interconnected in delay chain.
    Callables are processed through *group* ``DelayQueue``, then through *all* ``DelayQueue`` for
    group-type messages. For non-group messages, only the *all* ``DelayQueue`` is used.

    .. deprecated:: 13.3
       :class:`telegram.ext.MessageQueue` in its current form is deprecated and will be reinvented
       in a future release. See `this thread <https://git.io/JtDbF>`_ for a list of known bugs.

    Args:
        all_burst_limit (:obj:`int`, optional): Number of maximum *all-type* callbacks to process
            per time-window defined by :attr:`all_time_limit_ms`. Defaults to 30.
        all_time_limit_ms (:obj:`int`, optional): Defines width of *all-type* time-window used when
            each processing limit is calculated. Defaults to 1000 ms.
        group_burst_limit (:obj:`int`, optional): Number of maximum *group-type* callbacks to
            process per time-window defined by :attr:`group_time_limit_ms`. Defaults to 20.
        group_time_limit_ms (:obj:`int`, optional): Defines width of *group-type* time-window used
            when each processing limit is calculated. Defaults to 60000 ms.
        exc_route (:obj:`callable`, optional): A callable, accepting one positional argument; used
            to route exceptions from processor threads to main thread; is called on ``Exception``
            subclass exceptions. If not provided, exceptions are routed through dummy handler,
            which re-raises them.
        autostart (:obj:`bool`, optional): If :obj:`True`, processors are started immediately after
            object's creation; if :obj:`False`, should be started manually by :attr:`start` method.
            Defaults to :obj:`True`.

    """

    def __init__(
        self,
        all_burst_limit: int = 30,
        all_time_limit_ms: int = 1000,
        group_burst_limit: int = 20,
        group_time_limit_ms: int = 60000,
        exc_route: Callable[[Exception], None] = None,
        autostart: bool = True,
    ):
        warnings.warn(
            'MessageQueue in its current form is deprecated and will be reinvented in a future '
            'release. See https://git.io/JtDbF for a list of known bugs.',
            category=TelegramDeprecationWarning,
        )

        # create according delay queues, use composition
        self._all_delayq = DelayQueue(
            burst_limit=all_burst_limit,
            time_limit_ms=all_time_limit_ms,
            exc_route=exc_route,
            autostart=autostart,
        )
        self._group_delayq = DelayQueue(
            burst_limit=group_burst_limit,
            time_limit_ms=group_time_limit_ms,
            exc_route=exc_route,
            autostart=autostart,
        )

    def start(self) -> None:
        """Method is used to manually start the ``MessageQueue`` processing."""
        self._all_delayq.start()
        self._group_delayq.start()

    def stop(self, timeout: float = None) -> None:
        self._group_delayq.stop(timeout=timeout)
        self._all_delayq.stop(timeout=timeout)

    stop.__doc__ = DelayQueue.stop.__doc__ or ''  # reuse docstring if any

    def __call__(self, promise: Callable, is_group_msg: bool = False) -> Callable:
        """
        Processes callables in throughput-limiting queues to avoid hitting limits (specified with
        :attr:`burst_limit` and :attr:`time_limit`.

        Args:
            promise (:obj:`callable`): Mainly the ``telegram.utils.promise.Promise`` (see Notes for
                other callables), that is processed in delay queues.
            is_group_msg (:obj:`bool`, optional): Defines whether ``promise`` would be processed in
                group*+*all* ``DelayQueue``s (if set to :obj:`True`), or only through *all*
                ``DelayQueue`` (if set to :obj:`False`), resulting in needed delays to avoid
                hitting specified limits. Defaults to :obj:`False`.

        Note:
            Method is designed to accept ``telegram.utils.promise.Promise`` as ``promise``
            argument, but other callables could be used too. For example, lambdas or simple
            functions could be used to wrap original func to be called with needed args. In that
            case, be sure that either wrapper func does not raise outside exceptions or the proper
            :attr:`exc_route` handler is provided.

        Returns:
            :obj:`callable`: Used as ``promise`` argument.

        """

        if not is_group_msg:  # ignore middle group delay
            self._all_delayq(promise)
        else:  # use middle group delay
            self._group_delayq(self._all_delayq, promise)
        return promise


def queuedmessage(method: Callable) -> Callable:
    """A decorator to be used with :attr:`telegram.Bot` send* methods.

    Note:
        As it probably wouldn't be a good idea to make this decorator a property, it has been coded
        as decorator function, so it implies that first positional argument to wrapped MUST be
        self.

    The next object attributes are used by decorator:

    Attributes:
        self._is_messages_queued_default (:obj:`bool`): Value to provide class-defaults to
            ``queued`` kwarg if not provided during wrapped method call.
        self._msg_queue (:class:`telegram.ext.messagequeue.MessageQueue`): The actual
            ``MessageQueue`` used to delay outbound messages according to specified time-limits.

    Wrapped method starts accepting the next kwargs:

    Args:
        queued (:obj:`bool`, optional): If set to :obj:`True`, the ``MessageQueue`` is used to
            process output messages. Defaults to `self._is_queued_out`.
        isgroup (:obj:`bool`, optional): If set to :obj:`True`, the message is meant to be
            group-type(as there's no obvious way to determine its type in other way at the moment).
            Group-type messages could have additional processing delay according to limits set
            in `self._out_queue`. Defaults to :obj:`False`.

    Returns:
        ``telegram.utils.promise.Promise``: In case call is queued or original method's return
        value if it's not.

    """

    @functools.wraps(method)
    def wrapped(self: 'Bot', *args: object, **kwargs: object) -> object:
        # pylint: disable=W0212
        queued = kwargs.pop(
            'queued', self._is_messages_queued_default  # type: ignore[attr-defined]
        )
        isgroup = kwargs.pop('isgroup', False)
        if queued:
            prom = Promise(method, (self,) + args, kwargs)
            return self._msg_queue(prom, isgroup)  # type: ignore[attr-defined]
        return method(self, *args, **kwargs)

    return wrapped



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import Update, Message, Chat, User, TelegramError
from telegram.ext import CallbackContext


class TestCallbackContext:
    def test_non_context_dp(self, dp):
        with pytest.raises(ValueError):
            CallbackContext(dp)

    def test_from_job(self, cdp):
        job = cdp.job_queue.run_once(lambda x: x, 10)

        callback_context = CallbackContext.from_job(job, cdp)

        assert callback_context.job is job
        assert callback_context.chat_data is None
        assert callback_context.user_data is None
        assert callback_context.bot_data is cdp.bot_data
        assert callback_context.bot is cdp.bot
        assert callback_context.job_queue is cdp.job_queue
        assert callback_context.update_queue is cdp.update_queue

    def test_from_update(self, cdp):
        update = Update(
            0, message=Message(0, None, Chat(1, 'chat'), from_user=User(1, 'user', False))
        )

        callback_context = CallbackContext.from_update(update, cdp)

        assert callback_context.chat_data == {}
        assert callback_context.user_data == {}
        assert callback_context.bot_data is cdp.bot_data
        assert callback_context.bot is cdp.bot
        assert callback_context.job_queue is cdp.job_queue
        assert callback_context.update_queue is cdp.update_queue

        callback_context_same_user_chat = CallbackContext.from_update(update, cdp)

        callback_context.bot_data['test'] = 'bot'
        callback_context.chat_data['test'] = 'chat'
        callback_context.user_data['test'] = 'user'

        assert callback_context_same_user_chat.bot_data is callback_context.bot_data
        assert callback_context_same_user_chat.chat_data is callback_context.chat_data
        assert callback_context_same_user_chat.user_data is callback_context.user_data

        update_other_user_chat = Update(
            0, message=Message(0, None, Chat(2, 'chat'), from_user=User(2, 'user', False))
        )

        callback_context_other_user_chat = CallbackContext.from_update(update_other_user_chat, cdp)

        assert callback_context_other_user_chat.bot_data is callback_context.bot_data
        assert callback_context_other_user_chat.chat_data is not callback_context.chat_data
        assert callback_context_other_user_chat.user_data is not callback_context.user_data

    def test_from_update_not_update(self, cdp):
        callback_context = CallbackContext.from_update(None, cdp)

        assert callback_context.chat_data is None
        assert callback_context.user_data is None
        assert callback_context.bot_data is cdp.bot_data
        assert callback_context.bot is cdp.bot
        assert callback_context.job_queue is cdp.job_queue
        assert callback_context.update_queue is cdp.update_queue

        callback_context = CallbackContext.from_update('', cdp)

        assert callback_context.chat_data is None
        assert callback_context.user_data is None
        assert callback_context.bot_data is cdp.bot_data
        assert callback_context.bot is cdp.bot
        assert callback_context.job_queue is cdp.job_queue
        assert callback_context.update_queue is cdp.update_queue

    def test_from_error(self, cdp):
        error = TelegramError('test')

        update = Update(
            0, message=Message(0, None, Chat(1, 'chat'), from_user=User(1, 'user', False))
        )

        callback_context = CallbackContext.from_error(update, error, cdp)

        assert callback_context.error is error
        assert callback_context.chat_data == {}
        assert callback_context.user_data == {}
        assert callback_context.bot_data is cdp.bot_data
        assert callback_context.bot is cdp.bot
        assert callback_context.job_queue is cdp.job_queue
        assert callback_context.update_queue is cdp.update_queue
        assert callback_context.async_args is None
        assert callback_context.async_kwargs is None

    def test_from_error_async_params(self, cdp):
        error = TelegramError('test')

        args = [1, '2']
        kwargs = {'one': 1, 2: 'two'}

        callback_context = CallbackContext.from_error(
            None, error, cdp, async_args=args, async_kwargs=kwargs
        )

        assert callback_context.error is error
        assert callback_context.async_args is args
        assert callback_context.async_kwargs is kwargs

    def test_match(self, cdp):
        callback_context = CallbackContext(cdp)

        assert callback_context.match is None

        callback_context.matches = ['test', 'blah']

        assert callback_context.match == 'test'

    def test_data_assignment(self, cdp):
        update = Update(
            0, message=Message(0, None, Chat(1, 'chat'), from_user=User(1, 'user', False))
        )

        callback_context = CallbackContext.from_update(update, cdp)

        with pytest.raises(AttributeError):
            callback_context.bot_data = {"test": 123}
        with pytest.raises(AttributeError):
            callback_context.user_data = {}
        with pytest.raises(AttributeError):
            callback_context.chat_data = "test"

    def test_dispatcher_attribute(self, cdp):
        callback_context = CallbackContext(cdp)
        assert callback_context.dispatcher == cdp



#!/usr/bin/env python
# flake8: noqa: E501
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
from copy import deepcopy

import pytest

from telegram import (
    PassportData,
    PassportFile,
    Bot,
    File,
    PassportElementErrorSelfie,
    PassportElementErrorDataField,
    Credentials,
    TelegramDecryptionError,
)


# Note: All classes in telegram.credentials (except EncryptedCredentials) aren't directly tested
# here, although they are implicitly tested. Testing for those classes was too much work and not
# worth it.


RAW_PASSPORT_DATA = {
    'credentials': {
        'hash': 'qB4hz2LMcXYhglwz6EvXMMyI3PURisWLXl/iCmCXcSk=',
        'secret': 'O6x3X2JrLO1lUIhw48os1gaenDuZLhesoZMKXehZwtM3vsxOdtxHKWQyLNwtbyy4snYpARXDwf8f1QHNmQ/M1PwBQvk1ozrZBXb4a6k/iYj+P4v8Xw2M++CRHqZv0LaxFtyHOXnNYZ6dXpNeF0ZvYYTmm0FsYvK+/3/F6VDB3Oe6xWlXFLwaCCP/jA9i2+dKp6iq8NLOo4VnxenPKWWYz20RZ50MdAbS3UR+NCx4AHM2P5DEGrHNW0tMXJ+FG3jpVrit5BuCbB/eRgKxNGRWNxEGV5hun5MChdxKCEHCimnUA97h7MZdoTgYxkrfvDSZ/V89BnFXLdr87t/NLvVE0Q==',
        'data': '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',
    },
    'data': [
        {
            'data': 'QRfzWcCN4WncvRO3lASG+d+c5gzqXtoCinQ1PgtYiZMKXCksx9eB9Ic1bOt8C/un9/XaX220PjJSO7Kuba+nXXC51qTsjqP9rnLKygnEIWjKrfiDdklzgcukpRzFSjiOAvhy86xFJZ1PfPSrFATy/Gp1RydLzbrBd2ZWxZqXrxcMoA0Q2UTTFXDoCYerEAiZoD69i79tB/6nkLBcUUvN5d52gKd/GowvxWqAAmdO6l1N7jlo6aWjdYQNBAK1KHbJdbRZMJLxC1MqMuZXAYrPoYBRKr5xAnxDTmPn/LEZKLc3gwwZyEgR5x7e9jp5heM6IEMmsv3O/6SUeEQs7P0iVuRSPLMJLfDdwns8Tl3fF2M4IxKVovjCaOVW+yHKsADDAYQPzzH2RcrWVD0TP5I64mzpK64BbTOq3qm3Hn51SV9uA/+LvdGbCp7VnzHx4EdUizHsVyilJULOBwvklsrDRvXMiWmh34ZSR6zilh051tMEcRf0I+Oe7pIxVJd/KKfYA2Z/eWVQTCn5gMuAInQNXFSqDIeIqBX+wca6kvOCUOXB7J2uRjTpLaC4DM9s/sNjSBvFixcGAngt+9oap6Y45rQc8ZJaNN/ALqEJAmkphW8=',
            'type': 'personal_details',
        },
        {
            'reverse_side': {
                'file_date': 1534074942,
                'file_id': 'DgADBAADNQQAAtoagFPf4wwmFZdmyQI',
                'file_unique_id': 'adc3145fd2e84d95b64d68eaa22aa33e',
            },
            'translation': [
                {
                    'file_size': 28640,
                    'file_date': 1535630933,
                    'file_id': 'DgADBAADswMAAisqQVAmooP-kVgLgAI',
                    'file_unique_id': '52a90d53d6064bb58feb582acdc3a324',
                },
                {
                    'file_size': 28672,
                    'file_date': 1535630933,
                    'file_id': 'DgADBAAD1QMAAnrpQFBMZsT3HysjwwI',
                    'file_unique_id': '7285f864d168441ba1f7d02146250432',
                },
            ],
            'front_side': {
                'file_size': 28624,
                'file_date': 1534074942,
                'file_id': 'DgADBAADxwMAApnQgVPK2-ckL2eXVAI',
                'file_unique_id': 'd9d52a700cbb4a189a80104aa5978133',
            },
            'type': 'driver_license',
            'selfie': {
                'file_size': 28592,
                'file_date': 1534074942,
                'file_id': 'DgADBAADEQQAAkopgFNr6oi-wISRtAI',
                'file_unique_id': 'd4e390cca57b4da5a65322b304762a12',
            },
            'data': 'eJUOFuY53QKmGqmBgVWlLBAQCUQJ79n405SX6M5aGFIIodOPQqnLYvMNqTwTrXGDlW+mVLZcbu+y8luLVO8WsJB/0SB7q5WaXn/IMt1G9lz5G/KMLIZG/x9zlnimsaQLg7u8srG6L4KZzv+xkbbHjZdETrxU8j0N/DoS4HvLMRSJAgeFUrY6v2YW9vSRg+fSxIqQy1jR2VKpzAT8OhOz7A==',
        },
        {
            'translation': [
                {
                    'file_size': 28480,
                    'file_date': 1535630939,
                    'file_id': 'DgADBAADyQUAAqyqQVC_eoX_KwNjJwI',
                    'file_unique_id': '38b2877b443542cbaf520c6e36a33ac4',
                },
                {
                    'file_size': 28528,
                    'file_date': 1535630939,
                    'file_id': 'DgADBAADsQQAAubTQVDRO_FN3lOwWwI',
                    'file_unique_id': 'f008ca48c44b4a47895ddbcd2f76741e',
                },
            ],
            'files': [
                {
                    'file_size': 28640,
                    'file_date': 1534074988,
                    'file_id': 'DgADBAADLAMAAhwfgVMyfGa5Nr0LvAI',
                    'file_unique_id': 'b170748794834644baaa3ec57ee4ce7a',
                },
                {
                    'file_size': 28480,
                    'file_date': 1534074988,
                    'file_id': 'DgADBAADaQQAAsFxgVNVfLZuT-_3ZQI',
                    'file_unique_id': '19a12ae34dca424b85e0308f706cee75',
                },
            ],
            'type': 'utility_bill',
        },
        {
            'data': 'j9SksVkSj128DBtZA+3aNjSFNirzv+R97guZaMgae4Gi0oDVNAF7twPR7j9VSmPedfJrEwL3O889Ei+a5F1xyLLyEI/qEBljvL70GFIhYGitS0JmNabHPHSZrjOl8b4s/0Z0Px2GpLO5siusTLQonimdUvu4UPjKquYISmlKEKhtmGATy+h+JDjNCYuOkhakeNw0Rk0BHgj0C3fCb7WZNQSyVb+2GTu6caR6eXf/AFwFp0TV3sRz3h0WIVPW8bna',
            'type': 'address',
        },
        {'email': 'fb3e3i47zt@dispostable.com', 'type': 'email'},
    ],
}


@pytest.fixture(scope='function')
def all_passport_data():
    return [
        {'type': 'personal_details', 'data': RAW_PASSPORT_DATA['data'][0]['data']},
        {
            'type': 'passport',
            'data': RAW_PASSPORT_DATA['data'][1]['data'],
            'front_side': RAW_PASSPORT_DATA['data'][1]['front_side'],
            'selfie': RAW_PASSPORT_DATA['data'][1]['selfie'],
            'translation': RAW_PASSPORT_DATA['data'][1]['translation'],
        },
        {
            'type': 'internal_passport',
            'data': RAW_PASSPORT_DATA['data'][1]['data'],
            'front_side': RAW_PASSPORT_DATA['data'][1]['front_side'],
            'selfie': RAW_PASSPORT_DATA['data'][1]['selfie'],
            'translation': RAW_PASSPORT_DATA['data'][1]['translation'],
        },
        {
            'type': 'driver_license',
            'data': RAW_PASSPORT_DATA['data'][1]['data'],
            'front_side': RAW_PASSPORT_DATA['data'][1]['front_side'],
            'reverse_side': RAW_PASSPORT_DATA['data'][1]['reverse_side'],
            'selfie': RAW_PASSPORT_DATA['data'][1]['selfie'],
            'translation': RAW_PASSPORT_DATA['data'][1]['translation'],
        },
        {
            'type': 'identity_card',
            'data': RAW_PASSPORT_DATA['data'][1]['data'],
            'front_side': RAW_PASSPORT_DATA['data'][1]['front_side'],
            'reverse_side': RAW_PASSPORT_DATA['data'][1]['reverse_side'],
            'selfie': RAW_PASSPORT_DATA['data'][1]['selfie'],
            'translation': RAW_PASSPORT_DATA['data'][1]['translation'],
        },
        {
            'type': 'utility_bill',
            'files': RAW_PASSPORT_DATA['data'][2]['files'],
            'translation': RAW_PASSPORT_DATA['data'][2]['translation'],
        },
        {
            'type': 'bank_statement',
            'files': RAW_PASSPORT_DATA['data'][2]['files'],
            'translation': RAW_PASSPORT_DATA['data'][2]['translation'],
        },
        {
            'type': 'rental_agreement',
            'files': RAW_PASSPORT_DATA['data'][2]['files'],
            'translation': RAW_PASSPORT_DATA['data'][2]['translation'],
        },
        {
            'type': 'passport_registration',
            'files': RAW_PASSPORT_DATA['data'][2]['files'],
            'translation': RAW_PASSPORT_DATA['data'][2]['translation'],
        },
        {
            'type': 'temporary_registration',
            'files': RAW_PASSPORT_DATA['data'][2]['files'],
            'translation': RAW_PASSPORT_DATA['data'][2]['translation'],
        },
        {'type': 'address', 'data': RAW_PASSPORT_DATA['data'][3]['data']},
        {'type': 'email', 'email': 'fb3e3i47zt@dispostable.com'},
        {'type': 'phone_number', 'phone_number': 'fb3e3i47zt@dispostable.com'},
    ]


@pytest.fixture(scope='function')
def passport_data(bot):
    return PassportData.de_json(RAW_PASSPORT_DATA, bot=bot)


class TestPassport:
    driver_license_selfie_file_id = 'DgADBAADEQQAAkopgFNr6oi-wISRtAI'
    driver_license_selfie_file_unique_id = 'd4e390cca57b4da5a65322b304762a12'
    driver_license_front_side_file_id = 'DgADBAADxwMAApnQgVPK2-ckL2eXVAI'
    driver_license_front_side_file_unique_id = 'd9d52a700cbb4a189a80104aa5978133'
    driver_license_reverse_side_file_id = 'DgADBAADNQQAAtoagFPf4wwmFZdmyQI'
    driver_license_reverse_side_file_unique_id = 'adc3145fd2e84d95b64d68eaa22aa33e'
    driver_license_translation_1_file_id = 'DgADBAADswMAAisqQVAmooP-kVgLgAI'
    driver_license_translation_1_file_unique_id = '52a90d53d6064bb58feb582acdc3a324'
    driver_license_translation_2_file_id = 'DgADBAAD1QMAAnrpQFBMZsT3HysjwwI'
    driver_license_translation_2_file_unique_id = '7285f864d168441ba1f7d02146250432'
    utility_bill_1_file_id = 'DgADBAADLAMAAhwfgVMyfGa5Nr0LvAI'
    utility_bill_1_file_unique_id = 'b170748794834644baaa3ec57ee4ce7a'
    utility_bill_2_file_id = 'DgADBAADaQQAAsFxgVNVfLZuT-_3ZQI'
    utility_bill_2_file_unique_id = '19a12ae34dca424b85e0308f706cee75'
    utility_bill_translation_1_file_id = 'DgADBAADyQUAAqyqQVC_eoX_KwNjJwI'
    utility_bill_translation_1_file_unique_id = '38b2877b443542cbaf520c6e36a33ac4'
    utility_bill_translation_2_file_id = 'DgADBAADsQQAAubTQVDRO_FN3lOwWwI'
    utility_bill_translation_2_file_unique_id = 'f008ca48c44b4a47895ddbcd2f76741e'
    driver_license_selfie_credentials_file_hash = 'Cila/qLXSBH7DpZFbb5bRZIRxeFW2uv/ulL0u0JNsYI='
    driver_license_selfie_credentials_secret = 'tivdId6RNYNsvXYPppdzrbxOBuBOr9wXRPDcCvnXU7E='

    def test_creation(self, passport_data):
        assert isinstance(passport_data, PassportData)

    def test_expected_encrypted_values(self, passport_data):
        personal_details, driver_license, utility_bill, address, email = passport_data.data

        assert personal_details.type == 'personal_details'
        assert personal_details.data == RAW_PASSPORT_DATA['data'][0]['data']

        assert driver_license.type == 'driver_license'
        assert driver_license.data == RAW_PASSPORT_DATA['data'][1]['data']
        assert isinstance(driver_license.selfie, PassportFile)
        assert driver_license.selfie.file_id == self.driver_license_selfie_file_id
        assert driver_license.selfie.file_unique_id == self.driver_license_selfie_file_unique_id

        assert isinstance(driver_license.front_side, PassportFile)
        assert driver_license.front_side.file_id == self.driver_license_front_side_file_id
        assert (
            driver_license.front_side.file_unique_id
            == self.driver_license_front_side_file_unique_id
        )

        assert isinstance(driver_license.reverse_side, PassportFile)
        assert driver_license.reverse_side.file_id == self.driver_license_reverse_side_file_id
        assert (
            driver_license.reverse_side.file_unique_id
            == self.driver_license_reverse_side_file_unique_id
        )

        assert isinstance(driver_license.translation[0], PassportFile)
        assert driver_license.translation[0].file_id == self.driver_license_translation_1_file_id
        assert (
            driver_license.translation[0].file_unique_id
            == self.driver_license_translation_1_file_unique_id
        )

        assert isinstance(driver_license.translation[1], PassportFile)
        assert driver_license.translation[1].file_id == self.driver_license_translation_2_file_id
        assert (
            driver_license.translation[1].file_unique_id
            == self.driver_license_translation_2_file_unique_id
        )

        assert utility_bill.type == 'utility_bill'
        assert isinstance(utility_bill.files[0], PassportFile)
        assert utility_bill.files[0].file_id == self.utility_bill_1_file_id
        assert utility_bill.files[0].file_unique_id == self.utility_bill_1_file_unique_id

        assert isinstance(utility_bill.files[1], PassportFile)
        assert utility_bill.files[1].file_id == self.utility_bill_2_file_id
        assert utility_bill.files[1].file_unique_id == self.utility_bill_2_file_unique_id

        assert isinstance(utility_bill.translation[0], PassportFile)
        assert utility_bill.translation[0].file_id == self.utility_bill_translation_1_file_id
        assert (
            utility_bill.translation[0].file_unique_id
            == self.utility_bill_translation_1_file_unique_id
        )

        assert isinstance(utility_bill.translation[1], PassportFile)
        assert utility_bill.translation[1].file_id == self.utility_bill_translation_2_file_id
        assert (
            utility_bill.translation[1].file_unique_id
            == self.utility_bill_translation_2_file_unique_id
        )

        assert address.type == 'address'
        assert address.data == RAW_PASSPORT_DATA['data'][3]['data']

        assert email.type == 'email'
        assert email.email == 'fb3e3i47zt@dispostable.com'

    def test_expected_decrypted_values(self, passport_data):
        (
            personal_details,
            driver_license,
            utility_bill,
            address,
            email,
        ) = passport_data.decrypted_data

        assert personal_details.type == 'personal_details'
        assert personal_details.data.to_dict() == {
            'first_name': 'FIRSTNAME',
            'middle_name': 'MIDDLENAME',
            'first_name_native': 'FIRSTNAMENATIVE',
            'residence_country_code': 'DK',
            'birth_date': '01.01.2001',
            'last_name_native': 'LASTNAMENATIVE',
            'gender': 'female',
            'middle_name_native': 'MIDDLENAMENATIVE',
            'country_code': 'DK',
            'last_name': 'LASTNAME',
        }

        assert driver_license.type == 'driver_license'
        assert driver_license.data.to_dict() == {
            'expiry_date': '01.01.2001',
            'document_no': 'DOCUMENT_NO',
        }
        assert isinstance(driver_license.selfie, PassportFile)
        assert driver_license.selfie.file_id == self.driver_license_selfie_file_id
        assert driver_license.selfie.file_unique_id == self.driver_license_selfie_file_unique_id

        assert isinstance(driver_license.front_side, PassportFile)
        assert driver_license.front_side.file_id == self.driver_license_front_side_file_id
        assert (
            driver_license.front_side.file_unique_id
            == self.driver_license_front_side_file_unique_id
        )

        assert isinstance(driver_license.reverse_side, PassportFile)
        assert driver_license.reverse_side.file_id == self.driver_license_reverse_side_file_id
        assert (
            driver_license.reverse_side.file_unique_id
            == self.driver_license_reverse_side_file_unique_id
        )

        assert address.type == 'address'
        assert address.data.to_dict() == {
            'city': 'CITY',
            'street_line2': 'STREET_LINE2',
            'state': 'STATE',
            'post_code': 'POSTCODE',
            'country_code': 'DK',
            'street_line1': 'STREET_LINE1',
        }

        assert utility_bill.type == 'utility_bill'
        assert isinstance(utility_bill.files[0], PassportFile)
        assert utility_bill.files[0].file_id == self.utility_bill_1_file_id
        assert utility_bill.files[0].file_unique_id == self.utility_bill_1_file_unique_id

        assert isinstance(utility_bill.files[1], PassportFile)
        assert utility_bill.files[1].file_id == self.utility_bill_2_file_id
        assert utility_bill.files[1].file_unique_id == self.utility_bill_2_file_unique_id

        assert email.type == 'email'
        assert email.email == 'fb3e3i47zt@dispostable.com'

    def test_all_types(self, passport_data, bot, all_passport_data):
        credentials = passport_data.decrypted_credentials.to_dict()

        # Copy credentials from other types to all types so we can decrypt everything
        sd = credentials['secure_data']
        credentials['secure_data'] = {
            'personal_details': sd['personal_details'].copy(),
            'passport': sd['driver_license'].copy(),
            'internal_passport': sd['driver_license'].copy(),
            'driver_license': sd['driver_license'].copy(),
            'identity_card': sd['driver_license'].copy(),
            'address': sd['address'].copy(),
            'utility_bill': sd['utility_bill'].copy(),
            'bank_statement': sd['utility_bill'].copy(),
            'rental_agreement': sd['utility_bill'].copy(),
            'passport_registration': sd['utility_bill'].copy(),
            'temporary_registration': sd['utility_bill'].copy(),
        }

        new = PassportData.de_json(
            {
                'data': all_passport_data,
                # Replaced below
                'credentials': {'data': 'data', 'hash': 'hash', 'secret': 'secret'},
            },
            bot=bot,
        )

        new.credentials._decrypted_data = Credentials.de_json(credentials, bot)

        assert isinstance(new, PassportData)
        assert new.decrypted_data

    def test_bot_init_invalid_key(self, bot):
        with pytest.raises(TypeError):
            Bot(bot.token, private_key='Invalid key!')

        with pytest.raises(ValueError):
            Bot(bot.token, private_key=b'Invalid key!')

    def test_passport_data_okay_with_non_crypto_bot(self, bot):
        b = Bot(bot.token)
        assert PassportData.de_json(RAW_PASSPORT_DATA, bot=b)

    def test_wrong_hash(self, bot):
        data = deepcopy(RAW_PASSPORT_DATA)
        data['credentials']['hash'] = 'bm90Y29ycmVjdGhhc2g='  # Not correct hash
        passport_data = PassportData.de_json(data, bot=bot)
        with pytest.raises(TelegramDecryptionError):
            assert passport_data.decrypted_data

    def test_wrong_key(self, bot):
        short_key = b"-----BEGIN RSA PRIVATE KEY-----\r\nMIIBOQIBAAJBAKU+OZ2jJm7sCA/ec4gngNZhXYPu+DZ/TAwSMl0W7vAPXAsLplBk\r\nO8l6IBHx8N0ZC4Bc65mO3b2G8YAzqndyqH8CAwEAAQJAWOx3jQFzeVXDsOaBPdAk\r\nYTncXVeIc6tlfUl9mOLyinSbRNCy1XicOiOZFgH1rRKOGIC1235QmqxFvdecySoY\r\nwQIhAOFeGgeX9CrEPuSsd9+kqUcA2avCwqdQgSdy2qggRFyJAiEAu7QHT8JQSkHU\r\nDELfzrzc24AhjyG0z1DpGZArM8COascCIDK42SboXj3Z2UXiQ0CEcMzYNiVgOisq\r\nBUd5pBi+2mPxAiAM5Z7G/Sv1HjbKrOGh29o0/sXPhtpckEuj5QMC6E0gywIgFY6S\r\nNjwrAA+cMmsgY0O2fAzEKkDc5YiFsiXaGaSS4eA=\r\n-----END RSA PRIVATE KEY-----"
        b = Bot(bot.token, private_key=short_key)
        passport_data = PassportData.de_json(RAW_PASSPORT_DATA, bot=b)
        with pytest.raises(TelegramDecryptionError):
            assert passport_data.decrypted_data

        wrong_key = b"-----BEGIN RSA PRIVATE KEY-----\r\nMIIEogIBAAKCAQB4qCFltuvHakZze86TUweU7E/SB3VLGEHAe7GJlBmrou9SSWsL\r\nH7E++157X6UqWFl54LOE9MeHZnoW7rZ+DxLKhk6NwAHTxXPnvw4CZlvUPC3OFxg3\r\nhEmNen6ojSM4sl4kYUIa7F+Q5uMEYaboxoBen9mbj4zzMGsG4aY/xBOb2ewrXQyL\r\nRh//tk1Px4ago+lUPisAvQVecz7/6KU4Xj4Lpv2z20f3cHlZX6bb7HlE1vixCMOf\r\nxvfC5SkWEGZMR/ZoWQUsoDkrDSITF/S3GtLfg083TgtCKaOF3mCT27sJ1og77npP\r\n0cH/qdlbdoFtdrRj3PvBpaj/TtXRhmdGcJBxAgMBAAECggEAYSq1Sp6XHo8dkV8B\r\nK2/QSURNu8y5zvIH8aUrgqo8Shb7OH9bryekrB3vJtgNwR5JYHdu2wHttcL3S4SO\r\nftJQxbyHgmxAjHUVNGqOM6yPA0o7cR70J7FnMoKVgdO3q68pVY7ll50IET9/T0X9\r\nDrTdKFb+/eILFsXFS1NpeSzExdsKq3zM0sP/vlJHHYVTmZDGaGEvny/eLAS+KAfG\r\nrKP96DeO4C/peXEJzALZ/mG1ReBB05Qp9Dx1xEC20yreRk5MnnBA5oiHVG5ZLOl9\r\nEEHINidqN+TMNSkxv67xMfQ6utNu5IpbklKv/4wqQOJOO50HZ+qBtSurTN573dky\r\nzslbCQKBgQDHDUBYyKN/v69VLmvNVcxTgrOcrdbqAfefJXb9C3dVXhS8/oRkCRU/\r\ndzxYWNT7hmQyWUKor/izh68rZ/M+bsTnlaa7IdAgyChzTfcZL/2pxG9pq05GF1Q4\r\nBSJ896ZEe3jEhbpJXRlWYvz7455svlxR0H8FooCTddTmkU3nsQSx0wKBgQCbLSa4\r\nyZs2QVstQQerNjxAtLi0IvV8cJkuvFoNC2Q21oqQc7BYU7NJL7uwriprZr5nwkCQ\r\nOFQXi4N3uqimNxuSng31ETfjFZPp+pjb8jf7Sce7cqU66xxR+anUzVZqBG1CJShx\r\nVxN7cWN33UZvIH34gA2Ax6AXNnJG42B5Gn1GKwKBgQCZ/oh/p4nGNXfiAK3qB6yy\r\nFvX6CwuvsqHt/8AUeKBz7PtCU+38roI/vXF0MBVmGky+HwxREQLpcdl1TVCERpIT\r\nUFXThI9OLUwOGI1IcTZf9tby+1LtKvM++8n4wGdjp9qAv6ylQV9u09pAzZItMwCd\r\nUx5SL6wlaQ2y60tIKk0lfQKBgBJS+56YmA6JGzY11qz+I5FUhfcnpauDNGOTdGLT\r\n9IqRPR2fu7RCdgpva4+KkZHLOTLReoRNUojRPb4WubGfEk93AJju5pWXR7c6k3Bt\r\novS2mrJk8GQLvXVksQxjDxBH44sLDkKMEM3j7uYJqDaZNKbyoCWT7TCwikAau5qx\r\naRevAoGAAKZV705dvrpJuyoHFZ66luANlrAwG/vNf6Q4mBEXB7guqMkokCsSkjqR\r\nhsD79E6q06zA0QzkLCavbCn5kMmDS/AbA80+B7El92iIN6d3jRdiNZiewkhlWhEG\r\nm4N0gQRfIu+rUjsS/4xk8UuQUT/Ossjn/hExi7ejpKdCc7N++bc=\r\n-----END RSA PRIVATE KEY-----"
        b = Bot(bot.token, private_key=wrong_key)
        passport_data = PassportData.de_json(RAW_PASSPORT_DATA, bot=b)
        with pytest.raises(TelegramDecryptionError):
            assert passport_data.decrypted_data

    def test_mocked_download_passport_file(self, passport_data, monkeypatch):
        # The files are not coming from our test bot, therefore the file id is invalid/wrong
        # when coming from this bot, so we monkeypatch the call, to make sure that Bot.get_file
        # at least gets called
        # TODO: Actually download a passport file in a test
        selfie = passport_data.decrypted_data[1].selfie

        # NOTE: file_unique_id is not used in the get_file method, so it is passed directly
        def get_file(*_, **kwargs):
            return File(kwargs['file_id'], selfie.file_unique_id)

        monkeypatch.setattr(passport_data.bot, 'get_file', get_file)
        file = selfie.get_file()
        assert file.file_id == selfie.file_id
        assert file.file_unique_id == selfie.file_unique_id
        assert file._credentials.file_hash == self.driver_license_selfie_credentials_file_hash
        assert file._credentials.secret == self.driver_license_selfie_credentials_secret

    def test_mocked_set_passport_data_errors(self, monkeypatch, bot, chat_id, passport_data):
        def test(url, data, **kwargs):
            return (
                data['user_id'] == chat_id
                and data['errors'][0]['file_hash']
                == (
                    passport_data.decrypted_credentials.secure_data.driver_license.selfie.file_hash
                )
                and data['errors'][1]['data_hash']
                == (passport_data.decrypted_credentials.secure_data.driver_license.data.data_hash)
            )

        monkeypatch.setattr(bot.request, 'post', test)
        message = bot.set_passport_data_errors(
            chat_id,
            [
                PassportElementErrorSelfie(
                    'driver_license',
                    (
                        passport_data.decrypted_credentials.secure_data.driver_license.selfie.file_hash
                    ),
                    'You\'re not handsome enough to use this app!',
                ),
                PassportElementErrorDataField(
                    'driver_license',
                    'expiry_date',
                    (
                        passport_data.decrypted_credentials.secure_data.driver_license.data.data_hash
                    ),
                    'Your driver license is expired!',
                ),
            ],
        )
        assert message

    def test_de_json_and_to_dict(self, bot):
        passport_data = PassportData.de_json(RAW_PASSPORT_DATA, bot)
        assert passport_data.to_dict() == RAW_PASSPORT_DATA

        assert passport_data.decrypted_data
        assert passport_data.to_dict() == RAW_PASSPORT_DATA

    def test_equality(self, passport_data):
        a = PassportData(passport_data.data, passport_data.credentials)
        b = PassportData(passport_data.data, passport_data.credentials)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        passport_data.credentials.hash = 'NOTAPROPERHASH'
        c = PassportData(passport_data.data, passport_data.credentials)

        assert a != c
        assert hash(a) != hash(c)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import MessageEntity, Game, PhotoSize, Animation


@pytest.fixture(scope='function')
def game():
    return Game(
        TestGame.title,
        TestGame.description,
        TestGame.photo,
        text=TestGame.text,
        text_entities=TestGame.text_entities,
        animation=TestGame.animation,
    )


class TestGame:
    title = 'Python-telegram-bot Test Game'
    description = 'description'
    photo = [PhotoSize('Blah', 'ElseBlah', 640, 360, file_size=0)]
    text = (
        b'\\U0001f469\\u200d\\U0001f469\\u200d\\U0001f467'
        b'\\u200d\\U0001f467\\U0001f431http://google.com'
    ).decode('unicode-escape')
    text_entities = [MessageEntity(13, 17, MessageEntity.URL)]
    animation = Animation('blah', 'unique_id', 320, 180, 1)

    def test_de_json_required(self, bot):
        json_dict = {
            'title': self.title,
            'description': self.description,
            'photo': [self.photo[0].to_dict()],
        }
        game = Game.de_json(json_dict, bot)

        assert game.title == self.title
        assert game.description == self.description
        assert game.photo == self.photo

    def test_de_json_all(self, bot):
        json_dict = {
            'title': self.title,
            'description': self.description,
            'photo': [self.photo[0].to_dict()],
            'text': self.text,
            'text_entities': [self.text_entities[0].to_dict()],
            'animation': self.animation.to_dict(),
        }
        game = Game.de_json(json_dict, bot)

        assert game.title == self.title
        assert game.description == self.description
        assert game.photo == self.photo
        assert game.text == self.text
        assert game.text_entities == self.text_entities
        assert game.animation == self.animation

    def test_to_dict(self, game):
        game_dict = game.to_dict()

        assert isinstance(game_dict, dict)
        assert game_dict['title'] == game.title
        assert game_dict['description'] == game.description
        assert game_dict['photo'] == [game.photo[0].to_dict()]
        assert game_dict['text'] == game.text
        assert game_dict['text_entities'] == [game.text_entities[0].to_dict()]
        assert game_dict['animation'] == game.animation.to_dict()

    def test_parse_entity(self, game):
        entity = MessageEntity(type=MessageEntity.URL, offset=13, length=17)
        game.text_entities = [entity]

        assert game.parse_text_entity(entity) == 'http://google.com'

    def test_parse_entities(self, game):
        entity = MessageEntity(type=MessageEntity.URL, offset=13, length=17)
        entity_2 = MessageEntity(type=MessageEntity.BOLD, offset=13, length=1)
        game.text_entities = [entity_2, entity]

        assert game.parse_text_entities(MessageEntity.URL) == {entity: 'http://google.com'}
        assert game.parse_text_entities() == {entity: 'http://google.com', entity_2: 'h'}

    def test_equality(self):
        a = Game('title', 'description', [PhotoSize('Blah', 'unique_id', 640, 360, file_size=0)])
        b = Game(
            'title',
            'description',
            [PhotoSize('Blah', 'unique_id', 640, 360, file_size=0)],
            text='Here is a text',
        )
        c = Game(
            'eltit',
            'description',
            [PhotoSize('Blah', 'unique_id', 640, 360, file_size=0)],
            animation=Animation('blah', 'unique_id', 320, 180, 1),
        )
        d = Animation('blah', 'unique_id', 320, 180, 1)

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2020-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram.ext import Defaults
from telegram import User


class TestDefault:
    def test_data_assignment(self, cdp):
        defaults = Defaults()

        with pytest.raises(AttributeError):
            defaults.parse_mode = True
        with pytest.raises(AttributeError):
            defaults.explanation_parse_mode = True
        with pytest.raises(AttributeError):
            defaults.disable_notification = True
        with pytest.raises(AttributeError):
            defaults.disable_web_page_preview = True
        with pytest.raises(AttributeError):
            defaults.allow_sending_without_reply = True
        with pytest.raises(AttributeError):
            defaults.timeout = True
        with pytest.raises(AttributeError):
            defaults.quote = True
        with pytest.raises(AttributeError):
            defaults.tzinfo = True
        with pytest.raises(AttributeError):
            defaults.run_async = True

    def test_equality(self):
        a = Defaults(parse_mode='HTML', quote=True)
        b = Defaults(parse_mode='HTML', quote=True)
        c = Defaults(parse_mode='HTML', quote=False)
        d = Defaults(parse_mode='HTML', timeout=50)
        e = User(123, 'test_user', False)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import (
    InlineQueryResultVoice,
    InputTextMessageContent,
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    InlineQueryResultContact,
)


@pytest.fixture(scope='class')
def inline_query_result_contact():
    return InlineQueryResultContact(
        TestInlineQueryResultContact.id_,
        TestInlineQueryResultContact.phone_number,
        TestInlineQueryResultContact.first_name,
        last_name=TestInlineQueryResultContact.last_name,
        thumb_url=TestInlineQueryResultContact.thumb_url,
        thumb_width=TestInlineQueryResultContact.thumb_width,
        thumb_height=TestInlineQueryResultContact.thumb_height,
        input_message_content=TestInlineQueryResultContact.input_message_content,
        reply_markup=TestInlineQueryResultContact.reply_markup,
    )


class TestInlineQueryResultContact:
    id_ = 'id'
    type_ = 'contact'
    phone_number = 'phone_number'
    first_name = 'first_name'
    last_name = 'last_name'
    thumb_url = 'thumb url'
    thumb_width = 10
    thumb_height = 15
    input_message_content = InputTextMessageContent('input_message_content')
    reply_markup = InlineKeyboardMarkup([[InlineKeyboardButton('reply_markup')]])

    def test_expected_values(self, inline_query_result_contact):
        assert inline_query_result_contact.id == self.id_
        assert inline_query_result_contact.type == self.type_
        assert inline_query_result_contact.phone_number == self.phone_number
        assert inline_query_result_contact.first_name == self.first_name
        assert inline_query_result_contact.last_name == self.last_name
        assert inline_query_result_contact.thumb_url == self.thumb_url
        assert inline_query_result_contact.thumb_width == self.thumb_width
        assert inline_query_result_contact.thumb_height == self.thumb_height
        assert (
            inline_query_result_contact.input_message_content.to_dict()
            == self.input_message_content.to_dict()
        )
        assert inline_query_result_contact.reply_markup.to_dict() == self.reply_markup.to_dict()

    def test_to_dict(self, inline_query_result_contact):
        inline_query_result_contact_dict = inline_query_result_contact.to_dict()

        assert isinstance(inline_query_result_contact_dict, dict)
        assert inline_query_result_contact_dict['id'] == inline_query_result_contact.id
        assert inline_query_result_contact_dict['type'] == inline_query_result_contact.type
        assert (
            inline_query_result_contact_dict['phone_number']
            == inline_query_result_contact.phone_number
        )
        assert (
            inline_query_result_contact_dict['first_name']
            == inline_query_result_contact.first_name
        )
        assert (
            inline_query_result_contact_dict['last_name'] == inline_query_result_contact.last_name
        )
        assert (
            inline_query_result_contact_dict['thumb_url'] == inline_query_result_contact.thumb_url
        )
        assert (
            inline_query_result_contact_dict['thumb_width']
            == inline_query_result_contact.thumb_width
        )
        assert (
            inline_query_result_contact_dict['thumb_height']
            == inline_query_result_contact.thumb_height
        )
        assert (
            inline_query_result_contact_dict['input_message_content']
            == inline_query_result_contact.input_message_content.to_dict()
        )
        assert (
            inline_query_result_contact_dict['reply_markup']
            == inline_query_result_contact.reply_markup.to_dict()
        )

    def test_equality(self):
        a = InlineQueryResultContact(self.id_, self.phone_number, self.first_name)
        b = InlineQueryResultContact(self.id_, self.phone_number, self.first_name)
        c = InlineQueryResultContact(self.id_, '', self.first_name)
        d = InlineQueryResultContact('', self.phone_number, self.first_name)
        e = InlineQueryResultVoice(self.id_, '', '')

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import Update, User, ShippingAddress, ShippingQuery, Bot
from tests.conftest import check_shortcut_call, check_shortcut_signature, check_defaults_handling


@pytest.fixture(scope='class')
def shipping_query(bot):
    return ShippingQuery(
        TestShippingQuery.id_,
        TestShippingQuery.from_user,
        TestShippingQuery.invoice_payload,
        TestShippingQuery.shipping_address,
        bot=bot,
    )


class TestShippingQuery:
    id_ = 5
    invoice_payload = 'invoice_payload'
    from_user = User(0, '', False)
    shipping_address = ShippingAddress('GB', '', 'London', '12 Grimmauld Place', '', 'WC1')

    def test_de_json(self, bot):
        json_dict = {
            'id': TestShippingQuery.id_,
            'invoice_payload': TestShippingQuery.invoice_payload,
            'from': TestShippingQuery.from_user.to_dict(),
            'shipping_address': TestShippingQuery.shipping_address.to_dict(),
        }
        shipping_query = ShippingQuery.de_json(json_dict, bot)

        assert shipping_query.id == self.id_
        assert shipping_query.invoice_payload == self.invoice_payload
        assert shipping_query.from_user == self.from_user
        assert shipping_query.shipping_address == self.shipping_address
        assert shipping_query.bot is bot

    def test_to_dict(self, shipping_query):
        shipping_query_dict = shipping_query.to_dict()

        assert isinstance(shipping_query_dict, dict)
        assert shipping_query_dict['id'] == shipping_query.id
        assert shipping_query_dict['invoice_payload'] == shipping_query.invoice_payload
        assert shipping_query_dict['from'] == shipping_query.from_user.to_dict()
        assert shipping_query_dict['shipping_address'] == shipping_query.shipping_address.to_dict()

    def test_answer(self, monkeypatch, shipping_query):
        def make_assertion(*_, **kwargs):
            return kwargs['shipping_query_id'] == shipping_query.id

        assert check_shortcut_signature(
            ShippingQuery.answer, Bot.answer_shipping_query, ['shipping_query_id'], []
        )
        assert check_shortcut_call(
            shipping_query.answer, shipping_query.bot, 'answer_shipping_query'
        )
        assert check_defaults_handling(shipping_query.answer, shipping_query.bot)

        monkeypatch.setattr(shipping_query.bot, 'answer_shipping_query', make_assertion)
        assert shipping_query.answer(ok=True)

    def test_equality(self):
        a = ShippingQuery(self.id_, self.from_user, self.invoice_payload, self.shipping_address)
        b = ShippingQuery(self.id_, self.from_user, self.invoice_payload, self.shipping_address)
        c = ShippingQuery(self.id_, None, '', None)
        d = ShippingQuery(0, self.from_user, self.invoice_payload, self.shipping_address)
        e = Update(self.id_)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import MessageEntity, User


@pytest.fixture(scope="class", params=MessageEntity.ALL_TYPES)
def message_entity(request):
    type_ = request.param
    url = None
    if type_ == MessageEntity.TEXT_LINK:
        url = 't.me'
    user = None
    if type_ == MessageEntity.TEXT_MENTION:
        user = User(1, 'test_user', False)
    language = None
    if type == MessageEntity.PRE:
        language = "python"
    return MessageEntity(type, 1, 3, url=url, user=user, language=language)


class TestMessageEntity:
    type_ = 'url'
    offset = 1
    length = 2
    url = 'url'

    def test_de_json(self, bot):
        json_dict = {'type': self.type_, 'offset': self.offset, 'length': self.length}
        entity = MessageEntity.de_json(json_dict, bot)

        assert entity.type == self.type_
        assert entity.offset == self.offset
        assert entity.length == self.length

    def test_to_dict(self, message_entity):
        entity_dict = message_entity.to_dict()

        assert isinstance(entity_dict, dict)
        assert entity_dict['type'] == message_entity.type
        assert entity_dict['offset'] == message_entity.offset
        assert entity_dict['length'] == message_entity.length
        if message_entity.url:
            assert entity_dict['url'] == message_entity.url
        if message_entity.user:
            assert entity_dict['user'] == message_entity.user.to_dict()
        if message_entity.language:
            assert entity_dict['language'] == message_entity.language

    def test_equality(self):
        a = MessageEntity(MessageEntity.BOLD, 2, 3)
        b = MessageEntity(MessageEntity.BOLD, 2, 3)
        c = MessageEntity(MessageEntity.CODE, 2, 3)
        d = MessageEntity(MessageEntity.CODE, 5, 6)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram MessageEntity."""

from typing import TYPE_CHECKING, Any, List, Optional, ClassVar

from telegram import TelegramObject, User, constants
from telegram.utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class MessageEntity(TelegramObject):
    """
    This object represents one special entity in a text message. For example, hashtags,
    usernames, URLs, etc.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`type`, :attr:`offset` and :attr:`length` are equal.

    Args:
        type (:obj:`str`): Type of the entity. Can be mention (@username), hashtag, bot_command,
            url, email, phone_number, bold (bold text), italic (italic text), strikethrough,
            code (monowidth string), pre (monowidth block), text_link (for clickable text URLs),
            text_mention (for users without usernames).
        offset (:obj:`int`): Offset in UTF-16 code units to the start of the entity.
        length (:obj:`int`): Length of the entity in UTF-16 code units.
        url (:obj:`str`, optional): For :attr:`TEXT_LINK` only, url that will be opened after
            user taps on the text.
        user (:class:`telegram.User`, optional): For :attr:`TEXT_MENTION` only, the mentioned
             user.
        language (:obj:`str`, optional): For :attr:`PRE` only, the programming language of
            the entity text.

    Attributes:
        type (:obj:`str`): Type of the entity.
        offset (:obj:`int`): Offset in UTF-16 code units to the start of the entity.
        length (:obj:`int`): Length of the entity in UTF-16 code units.
        url (:obj:`str`): Optional. Url that will be opened after user taps on the text.
        user (:class:`telegram.User`): Optional. The mentioned user.
        language (:obj:`str`): Optional. Programming language of the entity text.

    """

    def __init__(
        self,
        type: str,  # pylint: disable=W0622
        offset: int,
        length: int,
        url: str = None,
        user: User = None,
        language: str = None,
        **_kwargs: Any,
    ):
        # Required
        self.type = type
        self.offset = offset
        self.length = length
        # Optionals
        self.url = url
        self.user = user
        self.language = language

        self._id_attrs = (self.type, self.offset, self.length)

    @classmethod
    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['MessageEntity']:
        data = cls.parse_data(data)

        if not data:
            return None

        data['user'] = User.de_json(data.get('user'), bot)

        return cls(**data)

    MENTION: ClassVar[str] = constants.MESSAGEENTITY_MENTION
    """:const:`telegram.constants.MESSAGEENTITY_MENTION`"""
    HASHTAG: ClassVar[str] = constants.MESSAGEENTITY_HASHTAG
    """:const:`telegram.constants.MESSAGEENTITY_HASHTAG`"""
    CASHTAG: ClassVar[str] = constants.MESSAGEENTITY_CASHTAG
    """:const:`telegram.constants.MESSAGEENTITY_CASHTAG`"""
    PHONE_NUMBER: ClassVar[str] = constants.MESSAGEENTITY_PHONE_NUMBER
    """:const:`telegram.constants.MESSAGEENTITY_PHONE_NUMBER`"""
    BOT_COMMAND: ClassVar[str] = constants.MESSAGEENTITY_BOT_COMMAND
    """:const:`telegram.constants.MESSAGEENTITY_BOT_COMMAND`"""
    URL: ClassVar[str] = constants.MESSAGEENTITY_URL
    """:const:`telegram.constants.MESSAGEENTITY_URL`"""
    EMAIL: ClassVar[str] = constants.MESSAGEENTITY_EMAIL
    """:const:`telegram.constants.MESSAGEENTITY_EMAIL`"""
    BOLD: ClassVar[str] = constants.MESSAGEENTITY_BOLD
    """:const:`telegram.constants.MESSAGEENTITY_BOLD`"""
    ITALIC: ClassVar[str] = constants.MESSAGEENTITY_ITALIC
    """:const:`telegram.constants.MESSAGEENTITY_ITALIC`"""
    CODE: ClassVar[str] = constants.MESSAGEENTITY_CODE
    """:const:`telegram.constants.MESSAGEENTITY_CODE`"""
    PRE: ClassVar[str] = constants.MESSAGEENTITY_PRE
    """:const:`telegram.constants.MESSAGEENTITY_PRE`"""
    TEXT_LINK: ClassVar[str] = constants.MESSAGEENTITY_TEXT_LINK
    """:const:`telegram.constants.MESSAGEENTITY_TEXT_LINK`"""
    TEXT_MENTION: ClassVar[str] = constants.MESSAGEENTITY_TEXT_MENTION
    """:const:`telegram.constants.MESSAGEENTITY_TEXT_MENTION`"""
    UNDERLINE: ClassVar[str] = constants.MESSAGEENTITY_UNDERLINE
    """:const:`telegram.constants.MESSAGEENTITY_UNDERLINE`"""
    STRIKETHROUGH: ClassVar[str] = constants.MESSAGEENTITY_STRIKETHROUGH
    """:const:`telegram.constants.MESSAGEENTITY_STRIKETHROUGH`"""
    ALL_TYPES: ClassVar[List[str]] = constants.MESSAGEENTITY_ALL_TYPES
    """:const:`telegram.constants.MESSAGEENTITY_ALL_TYPES`\n
    List of all the types"""



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
from pathlib import Path

import pytest
from flaky import flaky

from telegram import (
    InputMediaVideo,
    InputMediaPhoto,
    InputMediaAnimation,
    Message,
    InputFile,
    InputMediaAudio,
    InputMediaDocument,
    MessageEntity,
    ParseMode,
)

# noinspection PyUnresolvedReferences
from telegram.error import BadRequest
from .test_animation import animation, animation_file  # noqa: F401

# noinspection PyUnresolvedReferences
from .test_audio import audio, audio_file  # noqa: F401

# noinspection PyUnresolvedReferences
from .test_document import document, document_file  # noqa: F401

# noinspection PyUnresolvedReferences
from .test_photo import _photo, photo_file, photo, thumb  # noqa: F401

# noinspection PyUnresolvedReferences
from .test_video import video, video_file  # noqa: F401
from tests.conftest import expect_bad_request


@pytest.fixture(scope='class')
def input_media_video(class_thumb_file):
    return InputMediaVideo(
        media=TestInputMediaVideo.media,
        caption=TestInputMediaVideo.caption,
        width=TestInputMediaVideo.width,
        height=TestInputMediaVideo.height,
        duration=TestInputMediaVideo.duration,
        parse_mode=TestInputMediaVideo.parse_mode,
        caption_entities=TestInputMediaVideo.caption_entities,
        thumb=class_thumb_file,
        supports_streaming=TestInputMediaVideo.supports_streaming,
    )


@pytest.fixture(scope='class')
def input_media_photo(class_thumb_file):
    return InputMediaPhoto(
        media=TestInputMediaPhoto.media,
        caption=TestInputMediaPhoto.caption,
        parse_mode=TestInputMediaPhoto.parse_mode,
        caption_entities=TestInputMediaPhoto.caption_entities,
    )


@pytest.fixture(scope='class')
def input_media_animation(class_thumb_file):
    return InputMediaAnimation(
        media=TestInputMediaAnimation.media,
        caption=TestInputMediaAnimation.caption,
        parse_mode=TestInputMediaAnimation.parse_mode,
        caption_entities=TestInputMediaAnimation.caption_entities,
        width=TestInputMediaAnimation.width,
        height=TestInputMediaAnimation.height,
        thumb=class_thumb_file,
        duration=TestInputMediaAnimation.duration,
    )


@pytest.fixture(scope='class')
def input_media_audio(class_thumb_file):
    return InputMediaAudio(
        media=TestInputMediaAudio.media,
        caption=TestInputMediaAudio.caption,
        duration=TestInputMediaAudio.duration,
        performer=TestInputMediaAudio.performer,
        title=TestInputMediaAudio.title,
        thumb=class_thumb_file,
        parse_mode=TestInputMediaAudio.parse_mode,
        caption_entities=TestInputMediaAudio.caption_entities,
    )


@pytest.fixture(scope='class')
def input_media_document(class_thumb_file):
    return InputMediaDocument(
        media=TestInputMediaDocument.media,
        caption=TestInputMediaDocument.caption,
        thumb=class_thumb_file,
        parse_mode=TestInputMediaDocument.parse_mode,
        caption_entities=TestInputMediaDocument.caption_entities,
        disable_content_type_detection=TestInputMediaDocument.disable_content_type_detection,
    )


class TestInputMediaVideo:
    type_ = "video"
    media = "NOTAREALFILEID"
    caption = "My Caption"
    width = 3
    height = 4
    duration = 5
    parse_mode = 'HTML'
    supports_streaming = True
    caption_entities = [MessageEntity(MessageEntity.BOLD, 0, 2)]

    def test_expected_values(self, input_media_video):
        assert input_media_video.type == self.type_
        assert input_media_video.media == self.media
        assert input_media_video.caption == self.caption
        assert input_media_video.width == self.width
        assert input_media_video.height == self.height
        assert input_media_video.duration == self.duration
        assert input_media_video.parse_mode == self.parse_mode
        assert input_media_video.caption_entities == self.caption_entities
        assert input_media_video.supports_streaming == self.supports_streaming
        assert isinstance(input_media_video.thumb, InputFile)

    def test_to_dict(self, input_media_video):
        input_media_video_dict = input_media_video.to_dict()
        assert input_media_video_dict['type'] == input_media_video.type
        assert input_media_video_dict['media'] == input_media_video.media
        assert input_media_video_dict['caption'] == input_media_video.caption
        assert input_media_video_dict['width'] == input_media_video.width
        assert input_media_video_dict['height'] == input_media_video.height
        assert input_media_video_dict['duration'] == input_media_video.duration
        assert input_media_video_dict['parse_mode'] == input_media_video.parse_mode
        assert input_media_video_dict['caption_entities'] == [
            ce.to_dict() for ce in input_media_video.caption_entities
        ]
        assert input_media_video_dict['supports_streaming'] == input_media_video.supports_streaming

    def test_with_video(self, video):  # noqa: F811
        # fixture found in test_video
        input_media_video = InputMediaVideo(video, caption="test 3")
        assert input_media_video.type == self.type_
        assert input_media_video.media == video.file_id
        assert input_media_video.width == video.width
        assert input_media_video.height == video.height
        assert input_media_video.duration == video.duration
        assert input_media_video.caption == "test 3"

    def test_with_video_file(self, video_file):  # noqa: F811
        # fixture found in test_video
        input_media_video = InputMediaVideo(video_file, caption="test 3")
        assert input_media_video.type == self.type_
        assert isinstance(input_media_video.media, InputFile)
        assert input_media_video.caption == "test 3"

    def test_with_local_files(self):
        input_media_video = InputMediaVideo(
            'tests/data/telegram.mp4', thumb='tests/data/telegram.jpg'
        )
        assert input_media_video.media == (Path.cwd() / 'tests/data/telegram.mp4/').as_uri()
        assert input_media_video.thumb == (Path.cwd() / 'tests/data/telegram.jpg/').as_uri()


class TestInputMediaPhoto:
    type_ = "photo"
    media = "NOTAREALFILEID"
    caption = "My Caption"
    parse_mode = 'Markdown'
    caption_entities = [MessageEntity(MessageEntity.BOLD, 0, 2)]

    def test_expected_values(self, input_media_photo):
        assert input_media_photo.type == self.type_
        assert input_media_photo.media == self.media
        assert input_media_photo.caption == self.caption
        assert input_media_photo.parse_mode == self.parse_mode
        assert input_media_photo.caption_entities == self.caption_entities

    def test_to_dict(self, input_media_photo):
        input_media_photo_dict = input_media_photo.to_dict()
        assert input_media_photo_dict['type'] == input_media_photo.type
        assert input_media_photo_dict['media'] == input_media_photo.media
        assert input_media_photo_dict['caption'] == input_media_photo.caption
        assert input_media_photo_dict['parse_mode'] == input_media_photo.parse_mode
        assert input_media_photo_dict['caption_entities'] == [
            ce.to_dict() for ce in input_media_photo.caption_entities
        ]

    def test_with_photo(self, photo):  # noqa: F811
        # fixture found in test_photo
        input_media_photo = InputMediaPhoto(photo, caption="test 2")
        assert input_media_photo.type == self.type_
        assert input_media_photo.media == photo.file_id
        assert input_media_photo.caption == "test 2"

    def test_with_photo_file(self, photo_file):  # noqa: F811
        # fixture found in test_photo
        input_media_photo = InputMediaPhoto(photo_file, caption="test 2")
        assert input_media_photo.type == self.type_
        assert isinstance(input_media_photo.media, InputFile)
        assert input_media_photo.caption == "test 2"

    def test_with_local_files(self):
        input_media_photo = InputMediaPhoto('tests/data/telegram.mp4')
        assert input_media_photo.media == (Path.cwd() / 'tests/data/telegram.mp4/').as_uri()


class TestInputMediaAnimation:
    type_ = "animation"
    media = "NOTAREALFILEID"
    caption = "My Caption"
    parse_mode = 'Markdown'
    caption_entities = [MessageEntity(MessageEntity.BOLD, 0, 2)]
    width = 30
    height = 30
    duration = 1

    def test_expected_values(self, input_media_animation):
        assert input_media_animation.type == self.type_
        assert input_media_animation.media == self.media
        assert input_media_animation.caption == self.caption
        assert input_media_animation.parse_mode == self.parse_mode
        assert input_media_animation.caption_entities == self.caption_entities
        assert isinstance(input_media_animation.thumb, InputFile)

    def test_to_dict(self, input_media_animation):
        input_media_animation_dict = input_media_animation.to_dict()
        assert input_media_animation_dict['type'] == input_media_animation.type
        assert input_media_animation_dict['media'] == input_media_animation.media
        assert input_media_animation_dict['caption'] == input_media_animation.caption
        assert input_media_animation_dict['parse_mode'] == input_media_animation.parse_mode
        assert input_media_animation_dict['caption_entities'] == [
            ce.to_dict() for ce in input_media_animation.caption_entities
        ]
        assert input_media_animation_dict['width'] == input_media_animation.width
        assert input_media_animation_dict['height'] == input_media_animation.height
        assert input_media_animation_dict['duration'] == input_media_animation.duration

    def test_with_animation(self, animation):  # noqa: F811
        # fixture found in test_animation
        input_media_animation = InputMediaAnimation(animation, caption="test 2")
        assert input_media_animation.type == self.type_
        assert input_media_animation.media == animation.file_id
        assert input_media_animation.caption == "test 2"

    def test_with_animation_file(self, animation_file):  # noqa: F811
        # fixture found in test_animation
        input_media_animation = InputMediaAnimation(animation_file, caption="test 2")
        assert input_media_animation.type == self.type_
        assert isinstance(input_media_animation.media, InputFile)
        assert input_media_animation.caption == "test 2"

    def test_with_local_files(self):
        input_media_animation = InputMediaAnimation(
            'tests/data/telegram.mp4', thumb='tests/data/telegram.jpg'
        )
        assert input_media_animation.media == (Path.cwd() / 'tests/data/telegram.mp4').as_uri()
        assert input_media_animation.thumb == (Path.cwd() / 'tests/data/telegram.jpg').as_uri()


class TestInputMediaAudio:
    type_ = "audio"
    media = "NOTAREALFILEID"
    caption = "My Caption"
    duration = 3
    performer = 'performer'
    title = 'title'
    parse_mode = 'HTML'
    caption_entities = [MessageEntity(MessageEntity.BOLD, 0, 2)]

    def test_expected_values(self, input_media_audio):
        assert input_media_audio.type == self.type_
        assert input_media_audio.media == self.media
        assert input_media_audio.caption == self.caption
        assert input_media_audio.duration == self.duration
        assert input_media_audio.performer == self.performer
        assert input_media_audio.title == self.title
        assert input_media_audio.parse_mode == self.parse_mode
        assert input_media_audio.caption_entities == self.caption_entities
        assert isinstance(input_media_audio.thumb, InputFile)

    def test_to_dict(self, input_media_audio):
        input_media_audio_dict = input_media_audio.to_dict()
        assert input_media_audio_dict['type'] == input_media_audio.type
        assert input_media_audio_dict['media'] == input_media_audio.media
        assert input_media_audio_dict['caption'] == input_media_audio.caption
        assert input_media_audio_dict['duration'] == input_media_audio.duration
        assert input_media_audio_dict['performer'] == input_media_audio.performer
        assert input_media_audio_dict['title'] == input_media_audio.title
        assert input_media_audio_dict['parse_mode'] == input_media_audio.parse_mode
        assert input_media_audio_dict['caption_entities'] == [
            ce.to_dict() for ce in input_media_audio.caption_entities
        ]

    def test_with_audio(self, audio):  # noqa: F811
        # fixture found in test_audio
        input_media_audio = InputMediaAudio(audio, caption="test 3")
        assert input_media_audio.type == self.type_
        assert input_media_audio.media == audio.file_id
        assert input_media_audio.duration == audio.duration
        assert input_media_audio.performer == audio.performer
        assert input_media_audio.title == audio.title
        assert input_media_audio.caption == "test 3"

    def test_with_audio_file(self, audio_file):  # noqa: F811
        # fixture found in test_audio
        input_media_audio = InputMediaAudio(audio_file, caption="test 3")
        assert input_media_audio.type == self.type_
        assert isinstance(input_media_audio.media, InputFile)
        assert input_media_audio.caption == "test 3"

    def test_with_local_files(self):
        input_media_audio = InputMediaAudio(
            'tests/data/telegram.mp4', thumb='tests/data/telegram.jpg'
        )
        assert input_media_audio.media == (Path.cwd() / 'tests/data/telegram.mp4/').as_uri()
        assert input_media_audio.thumb == (Path.cwd() / 'tests/data/telegram.jpg/').as_uri()


class TestInputMediaDocument:
    type_ = "document"
    media = "NOTAREALFILEID"
    caption = "My Caption"
    parse_mode = 'HTML'
    caption_entities = [MessageEntity(MessageEntity.BOLD, 0, 2)]
    disable_content_type_detection = True

    def test_expected_values(self, input_media_document):
        assert input_media_document.type == self.type_
        assert input_media_document.media == self.media
        assert input_media_document.caption == self.caption
        assert input_media_document.parse_mode == self.parse_mode
        assert input_media_document.caption_entities == self.caption_entities
        assert (
            input_media_document.disable_content_type_detection
            == self.disable_content_type_detection
        )
        assert isinstance(input_media_document.thumb, InputFile)

    def test_to_dict(self, input_media_document):
        input_media_document_dict = input_media_document.to_dict()
        assert input_media_document_dict['type'] == input_media_document.type
        assert input_media_document_dict['media'] == input_media_document.media
        assert input_media_document_dict['caption'] == input_media_document.caption
        assert input_media_document_dict['parse_mode'] == input_media_document.parse_mode
        assert input_media_document_dict['caption_entities'] == [
            ce.to_dict() for ce in input_media_document.caption_entities
        ]
        assert (
            input_media_document['disable_content_type_detection']
            == input_media_document.disable_content_type_detection
        )

    def test_with_document(self, document):  # noqa: F811
        # fixture found in test_document
        input_media_document = InputMediaDocument(document, caption="test 3")
        assert input_media_document.type == self.type_
        assert input_media_document.media == document.file_id
        assert input_media_document.caption == "test 3"

    def test_with_document_file(self, document_file):  # noqa: F811
        # fixture found in test_document
        input_media_document = InputMediaDocument(document_file, caption="test 3")
        assert input_media_document.type == self.type_
        assert isinstance(input_media_document.media, InputFile)
        assert input_media_document.caption == "test 3"

    def test_with_local_files(self):
        input_media_document = InputMediaDocument(
            'tests/data/telegram.mp4', thumb='tests/data/telegram.jpg'
        )
        assert input_media_document.media == (Path.cwd() / 'tests/data/telegram.mp4').as_uri()
        assert input_media_document.thumb == (Path.cwd() / 'tests/data/telegram.jpg').as_uri()


@pytest.fixture(scope='function')  # noqa: F811
def media_group(photo, thumb):  # noqa: F811
    return [
        InputMediaPhoto(photo, caption='*photo* 1', parse_mode='Markdown'),
        InputMediaPhoto(thumb, caption='<b>photo</b> 2', parse_mode='HTML'),
        InputMediaPhoto(
            photo, caption='photo 3', caption_entities=[MessageEntity(MessageEntity.BOLD, 0, 5)]
        ),
    ]


class TestSendMediaGroup:
    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_send_media_group_photo(self, bot, chat_id, media_group):
        messages = bot.send_media_group(chat_id, media_group)
        assert isinstance(messages, list)
        assert len(messages) == 3
        assert all([isinstance(mes, Message) for mes in messages])
        assert all([mes.media_group_id == messages[0].media_group_id for mes in messages])
        assert all(mes.caption == f'photo {idx+1}' for idx, mes in enumerate(messages))
        assert all(
            mes.caption_entities == [MessageEntity(MessageEntity.BOLD, 0, 5)] for mes in messages
        )

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_send_media_group_all_args(self, bot, chat_id, media_group):
        m1 = bot.send_message(chat_id, text="test")
        messages = bot.send_media_group(
            chat_id, media_group, disable_notification=True, reply_to_message_id=m1.message_id
        )
        assert isinstance(messages, list)
        assert len(messages) == 3
        assert all([isinstance(mes, Message) for mes in messages])
        assert all([mes.media_group_id == messages[0].media_group_id for mes in messages])
        assert all(mes.caption == f'photo {idx+1}' for idx, mes in enumerate(messages))
        assert all(
            mes.caption_entities == [MessageEntity(MessageEntity.BOLD, 0, 5)] for mes in messages
        )

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_send_media_group_custom_filename(
        self,
        bot,
        chat_id,
        photo_file,  # noqa: F811
        animation_file,  # noqa: F811
        audio_file,  # noqa: F811
        video_file,  # noqa: F811
        monkeypatch,
    ):
        def make_assertion(url, data, **kwargs):
            result = all(im.media.filename == 'custom_filename' for im in data['media'])
            # We are a bit hacky here b/c Bot.send_media_group expects a list of Message-dicts
            return [Message(0, None, None, text=result).to_dict()]

        monkeypatch.setattr(bot.request, 'post', make_assertion)

        media = [
            InputMediaAnimation(animation_file, filename='custom_filename'),
            InputMediaAudio(audio_file, filename='custom_filename'),
            InputMediaPhoto(photo_file, filename='custom_filename'),
            InputMediaVideo(video_file, filename='custom_filename'),
        ]

        assert bot.send_media_group(chat_id, media)[0].text is True

    def test_send_media_group_with_thumbs(
        self, bot, chat_id, video_file, photo_file, monkeypatch  # noqa: F811
    ):
        def test(*args, **kwargs):
            data = kwargs['fields']
            video_check = data[input_video.media.attach] == input_video.media.field_tuple
            thumb_check = data[input_video.thumb.attach] == input_video.thumb.field_tuple
            result = video_check and thumb_check
            raise Exception(f"Test was {'successful' if result else 'failing'}")

        monkeypatch.setattr('telegram.utils.request.Request._request_wrapper', test)
        input_video = InputMediaVideo(video_file, thumb=photo_file)
        with pytest.raises(Exception, match='Test was successful'):
            bot.send_media_group(chat_id, [input_video, input_video])

    @flaky(3, 1)  # noqa: F811
    @pytest.mark.timeout(10)  # noqa: F811
    def test_send_media_group_new_files(
        self, bot, chat_id, video_file, photo_file, animation_file  # noqa: F811
    ):  # noqa: F811
        def func():
            with open('tests/data/telegram.jpg', 'rb') as file:
                return bot.send_media_group(
                    chat_id,
                    [
                        InputMediaVideo(video_file),
                        InputMediaPhoto(photo_file),
                        InputMediaPhoto(file.read()),
                    ],
                )

        messages = expect_bad_request(
            func, 'Type of file mismatch', 'Telegram did not accept the file.'
        )

        assert isinstance(messages, list)
        assert len(messages) == 3
        assert all([isinstance(mes, Message) for mes in messages])
        assert all([mes.media_group_id == messages[0].media_group_id for mes in messages])

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    @pytest.mark.parametrize(
        'default_bot,custom',
        [
            ({'allow_sending_without_reply': True}, None),
            ({'allow_sending_without_reply': False}, None),
            ({'allow_sending_without_reply': False}, True),
        ],
        indirect=['default_bot'],
    )
    def test_send_media_group_default_allow_sending_without_reply(
        self, default_bot, chat_id, media_group, custom
    ):
        reply_to_message = default_bot.send_message(chat_id, 'test')
        reply_to_message.delete()
        if custom is not None:
            messages = default_bot.send_media_group(
                chat_id,
                media_group,
                allow_sending_without_reply=custom,
                reply_to_message_id=reply_to_message.message_id,
            )
            assert [m.reply_to_message is None for m in messages]
        elif default_bot.defaults.allow_sending_without_reply:
            messages = default_bot.send_media_group(
                chat_id, media_group, reply_to_message_id=reply_to_message.message_id
            )
            assert [m.reply_to_message is None for m in messages]
        else:
            with pytest.raises(BadRequest, match='message not found'):
                default_bot.send_media_group(
                    chat_id, media_group, reply_to_message_id=reply_to_message.message_id
                )

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_edit_message_media(self, bot, chat_id, media_group):
        messages = bot.send_media_group(chat_id, media_group)
        cid = messages[-1].chat.id
        mid = messages[-1].message_id
        new_message = bot.edit_message_media(chat_id=cid, message_id=mid, media=media_group[0])
        assert isinstance(new_message, Message)

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    def test_edit_message_media_new_file(self, bot, chat_id, media_group, thumb_file):
        messages = bot.send_media_group(chat_id, media_group)
        cid = messages[-1].chat.id
        mid = messages[-1].message_id
        new_message = bot.edit_message_media(
            chat_id=cid, message_id=mid, media=InputMediaPhoto(thumb_file)
        )
        assert isinstance(new_message, Message)

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    @pytest.mark.parametrize(
        'default_bot', [{'parse_mode': ParseMode.HTML}], indirect=True, ids=['HTML-Bot']
    )
    @pytest.mark.parametrize('media_type', ['animation', 'document', 'audio', 'photo', 'video'])
    def test_edit_message_media_default_parse_mode(
        self,
        chat_id,
        default_bot,
        media_type,
        animation,  # noqa: F811
        document,  # noqa: F811
        audio,  # noqa: F811
        photo,  # noqa: F811
        video,  # noqa: F811
    ):
        html_caption = '<b>bold</b> <i>italic</i> <code>code</code>'
        markdown_caption = '*bold* _italic_ `code`'
        test_caption = 'bold italic code'
        test_entities = [
            MessageEntity(MessageEntity.BOLD, 0, 4),
            MessageEntity(MessageEntity.ITALIC, 5, 6),
            MessageEntity(MessageEntity.CODE, 12, 4),
        ]

        def build_media(parse_mode, med_type):
            kwargs = {}
            if parse_mode != ParseMode.HTML:
                kwargs['parse_mode'] = parse_mode
                kwargs['caption'] = markdown_caption
            else:
                kwargs['caption'] = html_caption

            if med_type == 'animation':
                return InputMediaAnimation(animation, **kwargs)
            if med_type == 'document':
                return InputMediaDocument(document, **kwargs)
            if med_type == 'audio':
                return InputMediaAudio(audio, **kwargs)
            if med_type == 'photo':
                return InputMediaPhoto(photo, **kwargs)
            if med_type == 'video':
                return InputMediaVideo(video, **kwargs)

        message = default_bot.send_photo(chat_id, photo)

        message = default_bot.edit_message_media(
            message.chat_id,
            message.message_id,
            media=build_media(parse_mode=ParseMode.HTML, med_type=media_type),
        )
        assert message.caption == test_caption
        assert message.caption_entities == test_entities

        # Remove caption to avoid "Message not changed"
        message.edit_caption()

        message = default_bot.edit_message_media(
            message.chat_id,
            message.message_id,
            media=build_media(parse_mode=ParseMode.MARKDOWN_V2, med_type=media_type),
        )
        assert message.caption == test_caption
        assert message.caption_entities == test_entities

        # Remove caption to avoid "Message not changed"
        message.edit_caption()

        message = default_bot.edit_message_media(
            message.chat_id,
            message.message_id,
            media=build_media(parse_mode=None, med_type=media_type),
        )
        assert message.caption == markdown_caption
        assert message.caption_entities == []



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].


class TestUtils:
    def test_promise_deprecation(self, recwarn):
        import telegram.utils.promise  # noqa: F401

        assert len(recwarn) == 1
        assert str(recwarn[0].message) == (
            'telegram.utils.promise is deprecated. Please use telegram.ext.utils.promise instead.'
        )

    def test_webhookhandler_deprecation(self, recwarn):
        import telegram.utils.webhookhandler  # noqa: F401

        assert len(recwarn) == 1
        assert str(recwarn[0].message) == (
            'telegram.utils.webhookhandler is deprecated. Please use '
            'telegram.ext.utils.webhookhandler instead.'
        )



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import (
    InlineQueryResultVoice,
    InputTextMessageContent,
    InlineKeyboardButton,
    InlineQueryResultVenue,
    InlineKeyboardMarkup,
)


@pytest.fixture(scope='class')
def inline_query_result_venue():
    return InlineQueryResultVenue(
        TestInlineQueryResultVenue.id_,
        TestInlineQueryResultVenue.latitude,
        TestInlineQueryResultVenue.longitude,
        TestInlineQueryResultVenue.title,
        TestInlineQueryResultVenue.address,
        foursquare_id=TestInlineQueryResultVenue.foursquare_id,
        foursquare_type=TestInlineQueryResultVenue.foursquare_type,
        thumb_url=TestInlineQueryResultVenue.thumb_url,
        thumb_width=TestInlineQueryResultVenue.thumb_width,
        thumb_height=TestInlineQueryResultVenue.thumb_height,
        input_message_content=TestInlineQueryResultVenue.input_message_content,
        reply_markup=TestInlineQueryResultVenue.reply_markup,
        google_place_id=TestInlineQueryResultVenue.google_place_id,
        google_place_type=TestInlineQueryResultVenue.google_place_type,
    )


class TestInlineQueryResultVenue:
    id_ = 'id'
    type_ = 'venue'
    latitude = 'latitude'
    longitude = 'longitude'
    title = 'title'
    address = 'address'
    foursquare_id = 'foursquare id'
    foursquare_type = 'foursquare type'
    google_place_id = 'google place id'
    google_place_type = 'google place type'
    thumb_url = 'thumb url'
    thumb_width = 10
    thumb_height = 15
    input_message_content = InputTextMessageContent('input_message_content')
    reply_markup = InlineKeyboardMarkup([[InlineKeyboardButton('reply_markup')]])

    def test_expected_values(self, inline_query_result_venue):
        assert inline_query_result_venue.id == self.id_
        assert inline_query_result_venue.type == self.type_
        assert inline_query_result_venue.latitude == self.latitude
        assert inline_query_result_venue.longitude == self.longitude
        assert inline_query_result_venue.title == self.title
        assert inline_query_result_venue.address == self.address
        assert inline_query_result_venue.foursquare_id == self.foursquare_id
        assert inline_query_result_venue.foursquare_type == self.foursquare_type
        assert inline_query_result_venue.google_place_id == self.google_place_id
        assert inline_query_result_venue.google_place_type == self.google_place_type
        assert inline_query_result_venue.thumb_url == self.thumb_url
        assert inline_query_result_venue.thumb_width == self.thumb_width
        assert inline_query_result_venue.thumb_height == self.thumb_height
        assert (
            inline_query_result_venue.input_message_content.to_dict()
            == self.input_message_content.to_dict()
        )
        assert inline_query_result_venue.reply_markup.to_dict() == self.reply_markup.to_dict()

    def test_to_dict(self, inline_query_result_venue):
        inline_query_result_venue_dict = inline_query_result_venue.to_dict()

        assert isinstance(inline_query_result_venue_dict, dict)
        assert inline_query_result_venue_dict['id'] == inline_query_result_venue.id
        assert inline_query_result_venue_dict['type'] == inline_query_result_venue.type
        assert inline_query_result_venue_dict['latitude'] == inline_query_result_venue.latitude
        assert inline_query_result_venue_dict['longitude'] == inline_query_result_venue.longitude
        assert inline_query_result_venue_dict['title'] == inline_query_result_venue.title
        assert inline_query_result_venue_dict['address'] == inline_query_result_venue.address
        assert (
            inline_query_result_venue_dict['foursquare_id']
            == inline_query_result_venue.foursquare_id
        )
        assert (
            inline_query_result_venue_dict['foursquare_type']
            == inline_query_result_venue.foursquare_type
        )
        assert (
            inline_query_result_venue_dict['google_place_id']
            == inline_query_result_venue.google_place_id
        )
        assert (
            inline_query_result_venue_dict['google_place_type']
            == inline_query_result_venue.google_place_type
        )
        assert inline_query_result_venue_dict['thumb_url'] == inline_query_result_venue.thumb_url
        assert (
            inline_query_result_venue_dict['thumb_width'] == inline_query_result_venue.thumb_width
        )
        assert (
            inline_query_result_venue_dict['thumb_height']
            == inline_query_result_venue.thumb_height
        )
        assert (
            inline_query_result_venue_dict['input_message_content']
            == inline_query_result_venue.input_message_content.to_dict()
        )
        assert (
            inline_query_result_venue_dict['reply_markup']
            == inline_query_result_venue.reply_markup.to_dict()
        )

    def test_equality(self):
        a = InlineQueryResultVenue(
            self.id_, self.longitude, self.latitude, self.title, self.address
        )
        b = InlineQueryResultVenue(
            self.id_, self.longitude, self.latitude, self.title, self.address
        )
        c = InlineQueryResultVenue(self.id_, '', self.latitude, self.title, self.address)
        d = InlineQueryResultVenue('', self.longitude, self.latitude, self.title, self.address)
        e = InlineQueryResultVoice(self.id_, '', '')

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest
from flaky import flaky

from telegram import Contact, Voice
from telegram.error import BadRequest


@pytest.fixture(scope='class')
def contact():
    return Contact(
        TestContact.phone_number,
        TestContact.first_name,
        TestContact.last_name,
        TestContact.user_id,
    )


class TestContact:
    phone_number = '+11234567890'
    first_name = 'Leandro'
    last_name = 'Toledo'
    user_id = 23

    def test_de_json_required(self, bot):
        json_dict = {'phone_number': self.phone_number, 'first_name': self.first_name}
        contact = Contact.de_json(json_dict, bot)

        assert contact.phone_number == self.phone_number
        assert contact.first_name == self.first_name

    def test_de_json_all(self, bot):
        json_dict = {
            'phone_number': self.phone_number,
            'first_name': self.first_name,
            'last_name': self.last_name,
            'user_id': self.user_id,
        }
        contact = Contact.de_json(json_dict, bot)

        assert contact.phone_number == self.phone_number
        assert contact.first_name == self.first_name
        assert contact.last_name == self.last_name
        assert contact.user_id == self.user_id

    def test_send_with_contact(self, monkeypatch, bot, chat_id, contact):
        def test(url, data, **kwargs):
            phone = data['phone_number'] == contact.phone_number
            first = data['first_name'] == contact.first_name
            last = data['last_name'] == contact.last_name
            return phone and first and last

        monkeypatch.setattr(bot.request, 'post', test)
        message = bot.send_contact(contact=contact, chat_id=chat_id)
        assert message

    @flaky(3, 1)
    @pytest.mark.timeout(10)
    @pytest.mark.parametrize(
        'default_bot,custom',
        [
            ({'allow_sending_without_reply': True}, None),
            ({'allow_sending_without_reply': False}, None),
            ({'allow_sending_without_reply': False}, True),
        ],
        indirect=['default_bot'],
    )
    def test_send_contact_default_allow_sending_without_reply(
        self, default_bot, chat_id, contact, custom
    ):
        reply_to_message = default_bot.send_message(chat_id, 'test')
        reply_to_message.delete()
        if custom is not None:
            message = default_bot.send_contact(
                chat_id,
                contact=contact,
                allow_sending_without_reply=custom,
                reply_to_message_id=reply_to_message.message_id,
            )
            assert message.reply_to_message is None
        elif default_bot.defaults.allow_sending_without_reply:
            message = default_bot.send_contact(
                chat_id, contact=contact, reply_to_message_id=reply_to_message.message_id
            )
            assert message.reply_to_message is None
        else:
            with pytest.raises(BadRequest, match='message not found'):
                default_bot.send_contact(
                    chat_id, contact=contact, reply_to_message_id=reply_to_message.message_id
                )

    def test_send_contact_without_required(self, bot, chat_id):
        with pytest.raises(ValueError, match='Either contact or phone_number and first_name'):
            bot.send_contact(chat_id=chat_id)

    def test_to_dict(self, contact):
        contact_dict = contact.to_dict()

        assert isinstance(contact_dict, dict)
        assert contact_dict['phone_number'] == contact.phone_number
        assert contact_dict['first_name'] == contact.first_name
        assert contact_dict['last_name'] == contact.last_name
        assert contact_dict['user_id'] == contact.user_id

    def test_equality(self):
        a = Contact(self.phone_number, self.first_name)
        b = Contact(self.phone_number, self.first_name)
        c = Contact(self.phone_number, '')
        d = Contact('', self.first_name)
        e = Voice('', 'unique_id', 0)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2021
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import os
from time import sleep, perf_counter

import pytest

import telegram.ext.messagequeue as mq


@pytest.mark.skipif(
    os.getenv('GITHUB_ACTIONS', False) and os.name == 'nt',
    reason="On windows precise timings are not accurate.",
)
class TestDelayQueue:
    N = 128
    burst_limit = 30
    time_limit_ms = 1000
    margin_ms = 0
    testtimes = []

    def call(self):
        self.testtimes.append(perf_counter())

    def test_delayqueue_limits(self):
        dsp = mq.DelayQueue(
            burst_limit=self.burst_limit, time_limit_ms=self.time_limit_ms, autostart=True
        )
        assert dsp.is_alive() is True

        for _ in range(self.N):
            dsp(self.call)

        starttime = perf_counter()
        # wait up to 20 sec more than needed
        app_endtime = (self.N * self.burst_limit / (1000 * self.time_limit_ms)) + starttime + 20
        while not dsp._queue.empty() and perf_counter() < app_endtime:
            sleep(1)
        assert dsp._queue.empty() is True  # check loop exit condition

        dsp.stop()
        assert dsp.is_alive() is False

        assert self.testtimes or self.N == 0
        passes, fails = [], []
        delta = (self.time_limit_ms - self.margin_ms) / 1000
        for start, stop in enumerate(range(self.burst_limit + 1, len(self.testtimes))):
            part = self.testtimes[start:stop]
            if (part[-1] - part[0]) >= delta:
                passes.append(part)
            else:
                fails.append(part)
        assert fails == []
